#!/usr/bin/env python
#encoding: utf8
#
# Copyright © Burak Arslan <burak at arskom dot com dot tr>,
#             Arskom Ltd. http://www.arskom.com.tr
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    1. Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#    2. Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#    3. Neither the name of the owner nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

import logging
import random
import sys
import base64

from pprint import pformat

from spyne.util.six.moves.http_cookies import SimpleCookie

# bcrypt seems to be among the latest consensus around cryptograpic circles on
# storing passwords.
# You need the package from http://code.google.com/p/py-bcrypt/
# You can install it by running easy_install py-bcrypt.
try:
    import bcrypt
except ImportError:
    print('easy_install --user py-bcrypt to get it.')
    raise

from spyne import Unicode, Application, rpc, Service
from spyne import M, ComplexModel, Fault, String
from spyne import ResourceNotFoundError
from spyne.protocol.soap import Soap11
from spyne.server.wsgi import WsgiApplication


class PublicKeyError(Fault):
    __namespace__ = 'spyne.examples.authentication'

    def __init__(self, value):
        super(PublicKeyError, self).__init__(
                faultcode='Client.KeyError',
                faultstring='Value %r not found' % value
            )


class AuthenticationError(Fault):
    __namespace__ = 'spyne.examples.authentication'

    def __init__(self, user_name):
        # TODO: self.transport.http.resp_code = HTTP_401

        super(AuthenticationError, self).__init__(
                faultcode='Client.AuthenticationError',
                faultstring='Invalid authentication request for %r' % user_name
            )


class AuthorizationError(Fault):
    __namespace__ = 'spyne.examples.authentication'

    def __init__(self):
        # TODO: self.transport.http.resp_code = HTTP_401

        super(AuthorizationError, self).__init__(
                faultcode='Client.AuthorizationError',
                faultstring='You are not authorized to access this resource.'
            )

class UnauthenticatedError(Fault):
    __namespace__ = 'spyne.examples.authentication'

    def __init__(self):
        super(UnauthenticatedError, self).__init__(
                faultcode='Client.UnauthenticatedError',
                faultstring='This resource can only be accessed after authentication.'
            )

class SpyneDict(dict):
    def __getitem__(self, key):
        try:
            return dict.__getitem__(self, key)
        except KeyError:
            raise PublicKeyError(key)


class Preferences(ComplexModel):
    __namespace__ = 'spyne.examples.authentication'

    language = String(max_len=2)
    time_zone = String


user_db = {
    'neo': bcrypt.hashpw(b'Wh1teR@bbit', bcrypt.gensalt()),
}

session_db = set()

preferences_db = SpyneDict({
    'neo': Preferences(language='en', time_zone='Underground/Zion'),
    'smith': Preferences(language='xx', time_zone='Matrix/Core'),
})


class Encoding:
    SESSION_ID = 'ascii'
    USER_NAME = PASSWORD = CREDENTIALS = 'utf8'


class UserService(Service):
    __tns__ = 'spyne.examples.authentication'

    @rpc(M(Unicode), M(Unicode),  _throws=AuthenticationError)
    def authenticate(ctx, user_name, password):
        ENC_C = Encoding.CREDENTIALS
        ENC_SID = Encoding.SESSION_ID

        password_hash = user_db.get(user_name, None)

        if password_hash is None:
           raise AuthenticationError(user_name)

        password_b = password.encode(ENC_C)
        if bcrypt.hashpw(password_b, password_hash) != password_hash:
           raise AuthenticationError(user_name)

        session_id = '%x' % (random.randint(1<<128, (1<<132)-1))
        session_key = (
            user_name.encode(ENC_C),
            session_id.encode(ENC_SID),
        )
        session_db.add(session_key)

        cookie = SimpleCookie()
        cookie["session-id"] = \
            base64.urlsafe_b64encode(b"\0".join(session_key)) \
                .decode('ascii')  # find out how to do urlsafe_b64encodestring

        cookie["session-id"]["max-age"] = 3600
        header_name, header_value = cookie.output().split(":", 1)
        ctx.transport.resp_headers[header_name] = header_value.strip()

        logging.debug("Response headers: %s", pformat(ctx.transport.resp_headers))


    @rpc(M(String), _throws=PublicKeyError, _returns=Preferences)
    def get_preferences(ctx, user_name):
        # Only allow access to the users own preferences.
        if user_name != ctx.udc:
            raise AuthorizationError()

        retval = preferences_db[user_name]

        return retval


def _on_method_call(ctx):
    if ctx.descriptor.name == "authenticate":
        # No checking of session cookie for call to authenticate
        return

    logging.debug("Request headers: %s", pformat(ctx.transport.req_env))

    cookie = SimpleCookie()
    http_cookie = ctx.transport.req_env.get("HTTP_COOKIE")
    if http_cookie:
        cookie.load(http_cookie)

    if "session-id" not in cookie:
        raise UnauthenticatedError()

    session_cookie = cookie["session-id"].value

    user_name, session_id = base64.urlsafe_b64decode(session_cookie) \
        .split(b"\0", 1)

    session_id = tuple(base64.urlsafe_b64decode(session_cookie).split(b"\0", 1))
    if not session_id in session_db:
        raise AuthenticationError(session_id[0])

    ctx.udc = session_id[0].decode(Encoding.USER_NAME)


UserService.event_manager.add_listener('method_call', _on_method_call)

if __name__=='__main__':
    from spyne.util.wsgi_wrapper import run_twisted

    logging.basicConfig(level=logging.DEBUG)
    logging.getLogger('spyne.protocol.xml').setLevel(logging.DEBUG)
    logging.getLogger('twisted').setLevel(logging.DEBUG)

    application = Application([UserService],
        tns='spyne.examples.authentication',
        in_protocol=Soap11(validator='lxml'),
        out_protocol=Soap11()
    )

    wsgi_app = WsgiApplication(application)
    wsgi_app.doc.wsdl11.xsl_href = "wsdl-viewer.xsl"

    twisted_apps = [
        (wsgi_app, b'app'),
    ]

    sys.exit(run_twisted(twisted_apps, 8000))



#!/usr/bin/env python
#
# spyne - Copyright (C) Spyne contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import logging
import unittest

from pprint import pprint
from lxml import etree

from spyne import Application
from spyne import rpc
from spyne.const import xml as ns
from spyne.const.xml import NS_XSD
from spyne.model import ByteArray
from spyne.model import ComplexModel
from spyne.model import XmlAttribute
from spyne.model import XmlData
from spyne.model import AnyXml
from spyne.model import Integer
from spyne.model import Mandatory as M
from spyne.model import Unicode
from spyne.model import Uuid
from spyne.model import Boolean
from spyne.protocol.soap import Soap11, Soap12
from spyne.service import Service
from spyne.util.xml import get_schema_documents
from spyne.util.xml import parse_schema_element
from spyne.util.xml import parse_schema_string

from spyne.interface.xml_schema import XmlSchema
from spyne.interface.xml_schema.genpy import CodeGenerator


class TestXmlSchema(unittest.TestCase):
    def test_choice_tag(self):
        class SomeObject(ComplexModel):
            __namespace__ = "badass_ns"

            one = Integer(xml_choice_group="numbers")
            two = Integer(xml_choice_group="numbers")
            punk = Unicode

        class KickassService(Service):
            @rpc(_returns=SomeObject)
            def wooo(ctx):
                return SomeObject()

        Application([KickassService],
            tns='kickass.ns',
            in_protocol=Soap11(validator='lxml'),
            out_protocol=Soap11()
        )

        docs = get_schema_documents([SomeObject])
        doc = docs['tns']
        print(etree.tostring(doc, pretty_print=True))
        assert len(doc.xpath('/xs:schema/xs:complexType[@name="SomeObject"]'
                                   '/xs:sequence/xs:element[@name="punk"]',
            namespaces={'xs': NS_XSD})) > 0
        assert len(doc.xpath('/xs:schema/xs:complexType[@name="SomeObject"]'
                    '/xs:sequence/xs:choice/xs:element[@name="one"]',
            namespaces={'xs': NS_XSD})) > 0

    def test_customized_class_with_empty_subclass(self):
        class SummaryStatsOfDouble(ComplexModel):
            _type_info = [('Min', XmlAttribute(Integer, use='required')),
                          ('Max', XmlAttribute(Integer, use='required')),
                          ('Avg', XmlAttribute(Integer, use='required'))]

        class SummaryStats(SummaryStatsOfDouble):
            ''' this is an empty base class '''

        class Payload(ComplexModel):
            _type_info = [('Stat1', SummaryStats.customize(nillable=False)),
                          ('Stat2', SummaryStats),
                          ('Stat3', SummaryStats),
                          ('Dummy', Unicode)]

        class JackedUpService(Service):
            @rpc(_returns=Payload)
            def GetPayload(ctx):
                return Payload()

        Application([JackedUpService],
            tns='kickass.ns',
            in_protocol=Soap11(validator='lxml'),
            out_protocol=Soap11()
        )

        # if no exceptions while building the schema, no problem.
        # see: https://github.com/arskom/spyne/issues/226


    def test_namespaced_xml_attribute(self):
        class Release(ComplexModel):
            __namespace__ = "http://usefulinc.com/ns/doap#"

            _type_info = [
                ('about', XmlAttribute(Unicode,
                             ns="http://www.w3.org/1999/02/22-rdf-syntax-ns#")),
            ]

        class Project(ComplexModel):
            __namespace__ = "http://usefulinc.com/ns/doap#"

            _type_info = [
                ('about', XmlAttribute(Unicode,
                             ns="http://www.w3.org/1999/02/22-rdf-syntax-ns#")),
                ('release', Release.customize(max_occurs=float('inf'))),
            ]

        class RdfService(Service):
            @rpc(Unicode, Unicode, _returns=Project)
            def some_call(ctx, a, b):
                pass

        Application([RdfService],
            tns='spynepi',
            in_protocol=Soap11(validator='lxml'),
            out_protocol=Soap11()
        )

        # if no exceptions while building the schema, no problem.

    def test_customized_simple_type_in_xml_attribute(self):
        class Product(ComplexModel):
            __namespace__ = 'some_ns'

            id = XmlAttribute(Uuid)
            edition = Unicode

        class SomeService(Service):
            @rpc(Product, _returns=Product)
            def echo_product(ctx, product):
                logging.info('edition_id: %r', product.edition_id)
                return product

        Application([SomeService], tns='some_ns',
            in_protocol=Soap11(validator='lxml'),
            out_protocol=Soap11()
        )

        # if no exceptions while building the schema, no problem.

    def test_binary_encodings(self):
        class Product(ComplexModel):
            __namespace__ = 'some_ns'

            hex = ByteArray(encoding='hex')
            base64_1 = ByteArray(encoding='base64')
            base64_2 = ByteArray

        class SomeService(Service):
            @rpc(Product, _returns=Product)
            def echo_product(ctx, product):
                logging.info('edition_id: %r', product.edition_id)
                return product

        app = Application([SomeService],
            tns='some_ns',
            in_protocol=Soap11(validator='lxml'),
            out_protocol=Soap11()
        )

        _ns = {'xs': NS_XSD}
        pref_xs = ns.PREFMAP[NS_XSD]
        xs = XmlSchema(app.interface)
        xs.build_interface_document()
        elt = xs.get_interface_document()['tns'].xpath(
                    '//xs:complexType[@name="Product"]',
                    namespaces=_ns)[0]

        assert elt.xpath('//xs:element[@name="base64_1"]/@type',
                            namespaces=_ns)[0] == '%s:base64Binary' % pref_xs
        assert elt.xpath('//xs:element[@name="base64_2"]/@type',
                            namespaces=_ns)[0] == '%s:base64Binary' % pref_xs
        assert elt.xpath('//xs:element[@name="hex"]/@type',
                            namespaces=_ns)[0] == '%s:hexBinary' % pref_xs

    def test_multilevel_customized_simple_type(self):
        class ExampleService(Service):
            __tns__ = 'http://xml.company.com/ns/example/'

            @rpc(M(Uuid), _returns=Unicode)
            def say_my_uuid(ctx, uuid):
                return 'Your UUID: %s' % uuid

        Application([ExampleService],
            tns='kickass.ns',
            in_protocol=Soap11(validator='lxml'),
            out_protocol=Soap11()
        )

        # if no exceptions while building the schema, no problem.
        # see: http://stackoverflow.com/questions/16042132/cannot-use-mandatory-uuid-or-other-pattern-related-must-be-type-as-rpc-argumen

    def test_any_tag(self):
        logging.basicConfig(level=logging.DEBUG)

        class SomeType(ComplexModel):
            __namespace__ = "zo"

            anything = AnyXml(schema_tag='{%s}any' % NS_XSD, namespace='##other',
                                                         process_contents='lax')

        docs = get_schema_documents([SomeType])
        print(etree.tostring(docs['tns'], pretty_print=True))
        _any = docs['tns'].xpath('//xsd:any', namespaces={'xsd': NS_XSD})

        assert len(_any) == 1
        assert _any[0].attrib['namespace'] == '##other'
        assert _any[0].attrib['processContents'] == 'lax'

    def _build_xml_data_test_schema(self, custom_root):
        tns = 'kickass.ns'

        class ProductEdition(ComplexModel):
            __namespace__ = tns
            id = XmlAttribute(Uuid)
            if custom_root:
                name = XmlData(Uuid)
            else:
                name = XmlData(Unicode)

        class Product(ComplexModel):
            __namespace__ = tns
            id = XmlAttribute(Uuid)
            edition = ProductEdition

        class ExampleService(Service):
            @rpc(Product, _returns=Product)
            def say_my_uuid(ctx, product):
                pass

        app = Application([ExampleService],
            tns='kickass.ns',
            in_protocol=Soap11(validator='lxml'),
            out_protocol=Soap11()
        )

        schema = XmlSchema(app.interface)
        schema.build_interface_document()
        schema.build_validation_schema()

        doc = schema.get_interface_document()['tns']
        print(etree.tostring(doc, pretty_print=True))
        return schema

    def test_xml_data_schema_doc(self):
        schema = self._build_xml_data_test_schema(custom_root=False)

        assert len(schema.get_interface_document()['tns'].xpath(
                '/xs:schema/xs:complexType[@name="ProductEdition"]'
                '/xs:simpleContent/xs:extension/xs:attribute[@name="id"]'
                ,namespaces={'xs': NS_XSD})) == 1

    def _test_xml_data_validation(self):
        schema = self._build_xml_data_test_schema(custom_root=False)

        assert schema.validation_schema.validate(etree.fromstring("""
            <Product id="00000000-0000-0000-0000-000000000000" xmlns="kickass.ns">
                <edition id="00000000-0000-0000-0000-000000000001">punk</edition>
            </Product>
        """)), schema.validation_schema.error_log.last_error

    def _test_xml_data_validation_custom_root(self):
        schema = self._build_xml_data_test_schema(custom_root=True)

        assert schema.validation_schema.validate(etree.fromstring("""
            <Product id="00000000-0000-0000-0000-000000000000" xmlns="kickass.ns">
                <edition id="00000000-0000-0000-0000-000000000001">
                    00000000-0000-0000-0000-000000000002
                </edition>
            </Product>
        """)), schema.validation_schema.error_log.last_error


    def test_subs(self):
        from lxml import etree
        from spyne.util.xml import get_schema_documents
        xpath = lambda o, x: o.xpath(x, namespaces={"xs": NS_XSD})

        m = {
            "s0": "aa",
            "s2": "cc",
            "s3": "dd",
        }

        class C(ComplexModel):
            __namespace__ = "aa"
            a = Integer
            b = Integer(sub_name="bb")
            c = Integer(sub_ns="cc")
            d = Integer(sub_ns="dd", sub_name="dd")

        elt = get_schema_documents([C], "aa")['tns']
        print(etree.tostring(elt, pretty_print=True))

        seq, = xpath(elt, "xs:complexType/xs:sequence")

        assert len(seq) == 4
        assert len(xpath(seq, 'xs:element[@name="a"]')) == 1
        assert len(xpath(seq, 'xs:element[@name="bb"]')) == 1

        # FIXME: this doesn't feel right.
        # check the spec to see whether it should it be prefixed.
        #
        #assert len(xpath(seq, 'xs:element[@name="{cc}c"]')) == 1
        #assert len(xpath(seq, 'xs:element[@name="{dd}dd"]')) == 1

    def test_mandatory(self):
        xpath = lambda o, x: o.xpath(x, namespaces={"xs": NS_XSD})

        class C(ComplexModel):
            __namespace__ = "aa"
            foo = XmlAttribute(M(Unicode))

        elt = get_schema_documents([C])['tns']
        print(etree.tostring(elt, pretty_print=True))
        foo, = xpath(elt, 'xs:complexType/xs:attribute[@name="foo"]')
        attrs = foo.attrib
        assert 'use' in attrs and attrs['use'] == 'required'

    def test_annotation(self):
        tns = 'some_ns'
        doc = "Some Doc"

        class SomeClass(ComplexModel):
            __namespace__ = tns
            some_attr = Unicode(doc=doc)

        schema = get_schema_documents([SomeClass], tns)['tns']
        print(etree.tostring(schema, pretty_print=True))
        assert schema.xpath("//xs:documentation/text()",
                                             namespaces={'xs': NS_XSD}) == [doc]


class TestParseOwnXmlSchema(unittest.TestCase):
    def test_simple(self):
        tns = 'some_ns'
        class SomeGuy(ComplexModel):
            __namespace__ = 'some_ns'

            id = Integer

        schema = get_schema_documents([SomeGuy], tns)['tns']
        print(etree.tostring(schema, pretty_print=True))

        objects = parse_schema_element(schema)
        pprint(objects[tns].types)

        NewGuy = objects[tns].types["SomeGuy"]
        assert NewGuy.get_type_name() == SomeGuy.get_type_name()
        assert NewGuy.get_namespace() == SomeGuy.get_namespace()
        assert dict(NewGuy._type_info) == dict(SomeGuy._type_info)

    def test_customized_unicode(self):
        tns = 'some_ns'
        class SomeGuy(ComplexModel):
            __namespace__ = tns
            name = Unicode(max_len=10, pattern="a", min_len=5, default="aa")

        schema = get_schema_documents([SomeGuy], tns)['tns']
        print(etree.tostring(schema, pretty_print=True))

        objects = parse_schema_element(schema)
        pprint(objects[tns].types)

        NewGuy = objects['some_ns'].types["SomeGuy"]
        assert NewGuy._type_info['name'].Attributes.max_len == 10
        assert NewGuy._type_info['name'].Attributes.min_len == 5
        assert NewGuy._type_info['name'].Attributes.pattern == "a"
        assert NewGuy._type_info['name'].Attributes.default == "aa"

    def test_boolean_default(self):
        tns = 'some_ns'
        class SomeGuy(ComplexModel):
            __namespace__ = tns
            bald = Boolean(default=True)

        schema = get_schema_documents([SomeGuy], tns)['tns']
        print(etree.tostring(schema, pretty_print=True))

        objects = parse_schema_element(schema)
        pprint(objects[tns].types)

        NewGuy = objects['some_ns'].types["SomeGuy"]
        assert NewGuy._type_info['bald'].Attributes.default == True

    def test_boolean_attribute_default(self):
        tns = 'some_ns'
        class SomeGuy(ComplexModel):
            __namespace__ = tns

            bald = XmlAttribute(Boolean(default=True))

        schema = get_schema_documents([SomeGuy], tns)['tns']
        print(etree.tostring(schema, pretty_print=True))

        objects = parse_schema_element(schema)
        pprint(objects[tns].types)

        NewGuy = objects['some_ns'].types["SomeGuy"]
        assert NewGuy._type_info['bald'].Attributes.default == True

    def test_attribute(self):
        tns = 'some_ns'
        class SomeGuy(ComplexModel):
            __namespace__ = tns

            name = XmlAttribute(Unicode)

        schema = get_schema_documents([SomeGuy], tns)['tns']
        print(etree.tostring(schema, pretty_print=True))

        objects = parse_schema_element(schema)
        pprint(objects)
        pprint(objects[tns].types)

        NewGuy = objects['some_ns'].types["SomeGuy"]
        assert NewGuy._type_info['name'].type is Unicode

    def test_attribute_with_customized_type(self):
        tns = 'some_ns'
        class SomeGuy(ComplexModel):
            __namespace__ = tns

            name = XmlAttribute(Unicode(default="aa"))

        schema = get_schema_documents([SomeGuy], tns)['tns']
        print(etree.tostring(schema, pretty_print=True))

        objects = parse_schema_element(schema)
        pprint(objects[tns].types)

        NewGuy = objects['some_ns'].types["SomeGuy"]
        assert NewGuy._type_info['name'].type.__orig__ is Unicode
        assert NewGuy._type_info['name'].type.Attributes.default == "aa"

    def test_inherited_attribute(self):
        class DeviceEntity(ComplexModel):
            token = XmlAttribute(Unicode, use='required')

        class DigitalInput(DeviceEntity):
            IdleState = XmlAttribute(Unicode)

        class SomeService(Service):
            @rpc(_returns=DigitalInput, _body_style='bare')
            def GetDigitalInput(ctx):
                return DigitalInput()

        Application([SomeService], 'some_tns',
            in_protocol=Soap11(validator='lxml'),
            out_protocol=Soap11())

    def test_simple_type_explicit_customization(self):
        class Header(ComplexModel):
            test = Boolean(min_occurs=0, nillable=False)
            pw = Unicode.customize(min_occurs=0, nillable=False, min_len=6)

        class Params(ComplexModel):
            sendHeader = Header.customize(nillable=False, min_occurs=1)

        class DummyService(Service):
            @rpc(Params, _returns=Unicode)
            def loadServices(ctx, serviceParams):
                return '42'

        Application([DummyService],
            tns='dummy',
            name='DummyService',
            in_protocol=Soap11(validator='lxml'),
            out_protocol=Soap11()
        )
        # if instantiation doesn't fail, test is green.


class TestParseForeignXmlSchema(unittest.TestCase):
    def test_simple_content(self):
        tns = 'some_ns'

        schema = """<xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema"
            targetNamespace="some_ns"
            elementFormDefault="qualified" attributeFormDefault="unqualified">
    <xsd:complexType name="SomeGuy">
        <xsd:simpleContent>
            <xsd:extension base="xsd:string">
                <xsd:attribute name="attr" type="xsd:string" use="optional"/>
            </xsd:extension>
        </xsd:simpleContent>
    </xsd:complexType>
</xsd:schema>"""

        objects = parse_schema_string(schema)
        pprint(objects[tns].types)

        NewGuy = objects[tns].types['SomeGuy']
        ti = NewGuy._type_info
        pprint(dict(ti))
        assert issubclass(ti['_data'], XmlData)
        assert ti['_data'].type is Unicode

        assert issubclass(ti['attr'], XmlAttribute)
        assert ti['attr'].type is Unicode


class TestCodeGeneration(unittest.TestCase):
    def _get_schema(self, *args):
        schema_doc = get_schema_documents(args)['tns']
        return parse_schema_element(schema_doc)

    def test_simple(self):
        ns = 'some_ns'

        class SomeObject(ComplexModel):
            __namespace__ = ns
            _type_info = [
                ('i', Integer),
                ('s', Unicode),
            ]

        s = self._get_schema(SomeObject)[ns]
        code = CodeGenerator().genpy(ns, s)

        # FIXME: Properly parse it
        assert """class SomeObject(_ComplexBase):
    _type_info = [
        ('i', Integer),
        ('s', Unicode),
    ]""" in code


if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python
# encoding: utf8
#
# Copyright © Burak Arslan <burak at arskom dot com dot tr>,
#             Arskom Ltd. http://www.arskom.com.tr
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    1. Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#    2. Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#    3. Neither the name of the owner nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

"""
This is a simple HelloWorld example to show the basics of writing a
MessagePack-via-vanilla-TCP-sockets server using the JsonDocument protocol.
"""


import logging

from spyne import Application, rpc, Service, Iterable, UnsignedInteger, \
    Unicode

from spyne.protocol.json import JsonDocument
from spyne.server.twisted.msgpack import TwistedMessagePackProtocolFactory
from spyne.server.wsgi import WsgiApplication


HOST = '127.0.0.1'
PORT = 5551


class HelloWorldService(Service):
    @rpc(Unicode, UnsignedInteger, _returns=Iterable(Unicode))
    def say_hello(ctx, name, times):
        for i in range(times):
            yield u'Hello, %s' % name


if __name__=='__main__':
    from twisted.internet import reactor
    from twisted.python import log

    # logging boilerplate
    logging.basicConfig(level=logging.DEBUG)
    logging.getLogger('spyne.protocol.xml').setLevel(logging.DEBUG)

    observer = log.PythonLoggingObserver('twisted')
    log.startLoggingWithObserver(observer.emit, setStdout=False)

    # set up application
    application = Application([HelloWorldService],
        'spyne.examples.hello.msgpack', in_protocol=JsonDocument(),
                                                    out_protocol=JsonDocument())

    # set up transport
    factory = TwistedMessagePackProtocolFactory(application)

    # set up listening endpoint
    reactor.listenTCP(PORT, factory, interface=HOST)

    # run the reactor
    reactor.run()



#!/usr/bin/env python
# encoding: utf8
#
# Copyright © Burak Arslan <burak at arskom dot com dot tr>,
#             Arskom Ltd. http://www.arskom.com.tr
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    1. Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#    2. Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#    3. Neither the name of the owner nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

from pprint import pprint

from spyne.client.http import HttpClient

from server_basic import application

c = HttpClient('http://localhost:8000/', application)

u = c.factory.create("User")

u.user_name = 'dave'
u.first_name = 'david'
u.last_name = 'smith'
u.email = 'david.smith@example.com'
u.permissions = []

permission = c.factory.create("Permission")
permission.application = 'table'
permission.operation = 'write'
u.permissions.append(permission)

permission = c.factory.create("Permission")
permission.application = 'table'
permission.operation = 'read'
u.permissions.append(permission)

retval = c.service.add_user(u)
print(retval)

pprint(c.service.get_user(retval))
pprint(list(c.service.get_all_user()))



#!/usr/bin/env python
#
# spyne - Copyright (C) Spyne contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import logging

from spyne.util import six

logging.basicConfig(level=logging.DEBUG)

import unittest

import msgpack

from spyne import MethodContext
from spyne.application import Application
from spyne.decorator import rpc
from spyne.decorator import srpc
from spyne.service import Service
from spyne.model.complex import Array
from spyne.model.primitive import String
from spyne.model.complex import ComplexModel
from spyne.model.primitive import Unicode
from spyne.protocol.msgpack import MessagePackDocument
from spyne.protocol.msgpack import MessagePackRpc
from spyne.util.six import BytesIO
from spyne.server import ServerBase
from spyne.server.wsgi import WsgiApplication
from spyne.test.protocol._test_dictdoc import TDictDocumentTest

from spyne.test.test_service import start_response


def convert_dict(d):
    if isinstance(d, six.text_type):
        return d.encode('utf8')

    if not isinstance(d, dict):
        return d

    r = {}

    for k, v in d.items():
        r[k.encode('utf8')] = convert_dict(v)

    return r


# apply spyne defaults to test unpacker
TestMessagePackDocument  = TDictDocumentTest(msgpack, MessagePackDocument,
                   loads_kwargs=dict(use_list=False), convert_dict=convert_dict)


class TestMessagePackRpc(unittest.TestCase):
    def test_invalid_input(self):
        class SomeService(Service):
            @srpc()
            def yay():
                pass

        app = Application([SomeService], 'tns',
                                in_protocol=MessagePackDocument(),
                                out_protocol=MessagePackDocument())

        server = ServerBase(app)

        initial_ctx = MethodContext(server, MethodContext.SERVER)
        initial_ctx.in_string = [b'\xdf']  # Invalid input
        ctx, = server.generate_contexts(initial_ctx)
        assert ctx.in_error.faultcode == 'Client.MessagePackDecodeError'

    def test_rpc(self):
        data = {"a":"b", "c": "d"}

        class KeyValuePair(ComplexModel):
            key = Unicode
            value = Unicode

        class SomeService(Service):
            @rpc(String(max_occurs='unbounded'),
                    _returns=Array(KeyValuePair),
                    _in_variable_names={
                        'keys': 'key'
                    }
                )
            def get_values(ctx, keys):
                for k in keys:
                    yield KeyValuePair(key=k, value=data[k])

        application = Application([SomeService],
            in_protocol=MessagePackRpc(),
            out_protocol=MessagePackRpc(ignore_wrappers=False),
            name='Service', tns='tns')
        server = WsgiApplication(application)

        input_string = msgpack.packb([0, 0, "get_values", [["a", "c"]]])
        input_stream = BytesIO(input_string)

        ret = server({
            'CONTENT_LENGTH': str(len(input_string)),
            'CONTENT_TYPE': 'application/x-msgpack',
            'HTTP_CONNECTION': 'close',
            'HTTP_CONTENT_LENGTH': str(len(input_string)),
            'HTTP_CONTENT_TYPE': 'application/x-msgpack',
            'PATH_INFO': '/',
            'QUERY_STRING': '',
            'SERVER_NAME': 'localhost',
            'SERVER_PORT': '7000',
            'REQUEST_METHOD': 'POST',
            'wsgi.url_scheme': 'http',
            'wsgi.input': input_stream,
        }, start_response)

        ret = b''.join(ret)
        print(repr(ret))
        ret = msgpack.unpackb(ret)
        print(repr(ret))

        s = [1, 0, None, {b'get_valuesResponse': {
            b'get_valuesResult': [
                  {b"KeyValuePair": {b'key': b'a', b'value': b'b'}},
                  {b"KeyValuePair": {b'key': b'c', b'value': b'd'}},
                ]
            }}
        ]
        print(s)
        assert ret == s


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

from suds.client import Client
import base64

# Suds does not support base64binary type, so we do the encoding manually.
file_data = base64.b64encode('file_data')

c=Client('http://localhost:9000/filemgr/?wsdl')
c.service.add('x', 'y', 'file_name', file_data)

print('file written.')
print()

print('incoming data:')
return_data = c.service.get('file_name')
print(repr(return_data))




#
# spyne - Copyright (C) Spyne contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

from __future__ import print_function

import logging
logger = logging.getLogger(__name__)

import re
RE_HTTP_ARRAY_INDEX = re.compile("\\[([0-9]+)\\]")

from mmap import mmap
from collections import defaultdict

from spyne.util import six
from spyne.util.six.moves.collections_abc import Iterable as AbcIterable

from spyne.error import ValidationError
from spyne.error import ResourceNotFoundError

from spyne.model import ByteArray, File, Fault, ComplexModelBase, Array, Any, \
    AnyDict, Uuid, Unicode

from spyne.protocol.dictdoc import DictDocument


class HierDictDocument(DictDocument):
    """This protocol contains logic for protocols that serialize and deserialize
    hierarchical dictionaries. Examples include: Json, MessagePack and Yaml.

    Implement ``create_in_document()`` and ``create_out_string()`` to use this.
    """

    VALID_UNICODE_SOURCES = (six.text_type, six.binary_type, memoryview,
                                                                mmap, bytearray)

    from_serstr = DictDocument.from_unicode
    to_serstr = DictDocument.to_unicode

    def get_class_name(self, cls):
        class_name = cls.get_type_name()
        if not six.PY2:
            if isinstance(class_name, bytes):
                class_name = class_name.decode('utf8')

        return class_name

    def get_complex_as(self, attr):
        if attr.complex_as is None:
            return self.complex_as
        return attr.complex_as

    def deserialize(self, ctx, message):
        assert message in (self.REQUEST, self.RESPONSE)

        self.event_manager.fire_event('before_deserialize', ctx)

        if ctx.descriptor is None:
            raise ResourceNotFoundError(ctx.method_request_string)

        # instantiate the result message
        if message is self.REQUEST:
            body_class = ctx.descriptor.in_message
        elif message is self.RESPONSE:
            body_class = ctx.descriptor.out_message
        else:
            raise ValueError(message)  # should be impossible

        if body_class:
            # assign raw result to its wrapper, result_message
            doc = ctx.in_body_doc

            logger.debug("Request: %r", doc)

            class_name = self.get_class_name(body_class)
            if self.ignore_wrappers:
                doc = doc.get(class_name, None)

            result_message = self._doc_to_object(ctx, body_class, doc,
                                                                 self.validator)
            ctx.in_object = result_message

        else:
            ctx.in_object = []

        self.event_manager.fire_event('after_deserialize', ctx)

    def _fault_to_doc(self, inst, cls=None):
        if cls is None:
            cls = Fault

        if self.complex_as is list:
            return [cls.to_list(inst.__class__, inst, self)]

        elif self.complex_as is tuple:
            fault_as_list = [Fault.to_list(inst.__class__, inst, self)]
            return tuple(fault_as_list)

        else:
            return [Fault.to_dict(inst.__class__, inst, self)]

    def serialize(self, ctx, message):
        assert message in (self.REQUEST, self.RESPONSE)

        self.event_manager.fire_event('before_serialize', ctx)

        if ctx.out_error is not None:
            ctx.out_document = self._fault_to_doc(ctx.out_error)
            return

        # get the result message
        if message is self.REQUEST:
            out_type = ctx.descriptor.in_message

        elif message is self.RESPONSE:
            out_type = ctx.descriptor.out_message

        else:
            assert False

        if out_type is None:
            return

        # assign raw result to its wrapper, result_message
        if ctx.descriptor.is_out_bare():
            out_instance, = ctx.out_object

        else:
            out_type_info = out_type.get_flat_type_info(out_type)

            # instantiate the result message
            out_instance = out_type()

            for i, (k, v) in enumerate(out_type_info.items()):
                attrs = self.get_cls_attrs(v)
                out_instance._safe_set(k, ctx.out_object[i], v, attrs)

        ctx.out_document = self._object_to_doc(out_type, out_instance, set()),

        logger.debug("Response: %r", ctx.out_document)
        self.event_manager.fire_event('after_serialize', ctx)

    def validate(self, key, cls, inst):
        if inst is None and self.get_cls_attrs(cls).nullable:
            pass

        elif issubclass(cls, Unicode) and not isinstance(inst,
                                                    self.VALID_UNICODE_SOURCES):
            raise ValidationError([key, inst])

    def _from_dict_value(self, ctx, key, cls, inst, validator):
        if validator is self.SOFT_VALIDATION:
            self.validate(key, cls, inst)

        cls_attrs = self.get_cls_attrs(cls)
        complex_as = self.get_complex_as(cls_attrs)
        if complex_as is list or complex_as is tuple:
            check_complex_as = (list, tuple)
        else:
            check_complex_as = complex_as

        # get native type
        if issubclass(cls, File):
            if isinstance(inst, check_complex_as):
                cls = cls_attrs.type or cls
                inst = self._parse(cls_attrs, inst)
                retval = self._doc_to_object(ctx, cls, inst, validator)

            else:
                retval = self.from_serstr(cls, inst, self.binary_encoding)

        else:
            inst = self._parse(cls_attrs, inst)

            if issubclass(cls, (Any, AnyDict)):
                retval = inst

            elif issubclass(cls, ComplexModelBase):
                retval = self._doc_to_object(ctx, cls, inst, validator)

            else:
                if cls_attrs.empty_is_none and inst in (u'', b''):
                    inst = None

                if (validator is self.SOFT_VALIDATION
                                        and isinstance(inst, six.string_types)
                                        and not cls.validate_string(cls, inst)):
                    raise ValidationError([key, inst])

                if issubclass(cls, (ByteArray, Uuid)):
                    retval = self.from_serstr(cls, inst, self.binary_encoding)

                elif issubclass(cls, Unicode):
                    if isinstance(inst, bytearray):
                        retval = six.text_type(inst,
                                encoding=cls_attrs.encoding or 'ascii',
                                                errors=cls_attrs.unicode_errors)

                    elif isinstance(inst, memoryview):
                        # FIXME: memoryview needs a .decode() function to avoid
                        #        needless copying here
                        retval = inst.tobytes().decode(
                            cls_attrs.encoding or 'ascii',
                                                errors=cls_attrs.unicode_errors)

                    elif isinstance(inst, mmap):
                        # FIXME: mmap needs a .decode() function to avoid
                        #        needless copying here
                        retval = mmap[:].decode(cls_attrs.encoding,
                                                errors=cls_attrs.unicode_errors)

                    elif isinstance(inst, six.binary_type):
                        retval = self.unicode_from_bytes(cls, inst)

                    else:
                        retval = inst

                else:
                    retval = self.from_serstr(cls, inst)

        # validate native type
        if validator is self.SOFT_VALIDATION:
            if not cls.validate_native(cls, retval):
                raise ValidationError([key, retval])

        return retval

    def _doc_to_object(self, ctx, cls, doc, validator=None):
        if doc is None:
            return []

        if issubclass(cls, Any):
            doc = self._cast(self.get_cls_attrs(cls), doc)
            return doc

        if issubclass(cls, Array):
            doc = self._cast(self.get_cls_attrs(cls), doc)
            retval = []
            (serializer,) = cls._type_info.values()

            if not isinstance(doc, AbcIterable):
                raise ValidationError(doc)

            for i, child in enumerate(doc):
                retval.append(self._from_dict_value(ctx, i, serializer, child,
                                                                     validator))

            return retval

        cls_attrs = self.get_cls_attrs(cls)
        if not self.ignore_wrappers and not cls_attrs.not_wrapped:
            if not isinstance(doc, dict):
                raise ValidationError(doc, "Wrapper documents must be dicts")

            if len(doc) == 0:
                return None

            if len(doc) > 1:
                raise ValidationError(doc, "There can be only one entry in a "
                                                                 "wrapper dict")

            subclasses = cls.get_subclasses()
            (class_name, doc), = doc.items()
            if not six.PY2 and isinstance(class_name, bytes):
                class_name = class_name.decode('utf8')

            if cls.get_type_name() != class_name and subclasses is not None \
                                                        and len(subclasses) > 0:
                for subcls in subclasses:
                    if subcls.get_type_name() == class_name:
                        break
                else:
                    raise ValidationError(class_name,
                        "Class name %%r is not registered as a subclass of %r" %
                                                            cls.get_type_name())

                if not self.issubclass(subcls, cls):
                    raise ValidationError(class_name,
                             "Class name %%r is not a subclass of %r" %
                                                            cls.get_type_name())
                cls = subcls

        inst = cls.get_deserialization_instance(ctx)

        # get all class attributes, including the ones coming from
        # parent classes.
        flat_type_info = cls.get_flat_type_info(cls)
        if flat_type_info is None:
            logger.critical("No flat_type_info found for type %r", cls)
            raise TypeError(cls)

        # this is for validating cls.Attributes.{min,max}_occurs
        frequencies = defaultdict(int)

        try:
            items = doc.items()
        except AttributeError:
            # Input is not a dict, so we assume it's a sequence that we can pair
            # with the incoming sequence with field names.
            # TODO: cache this
            try:
                items = zip([k for k, v in flat_type_info.items()
                                         if not self.get_cls_attrs(v).exc], doc)
            except TypeError as e:
                logger.error("Invalid document %r for %r", doc, cls)
                raise ValidationError(doc)

        # parse input to set incoming data to related attributes.
        for k, v in items:
            if self.key_encoding is not None and isinstance(k, bytes):
                try:
                    k = k.decode(self.key_encoding)
                except UnicodeDecodeError:
                    raise ValidationError(k)

            member = flat_type_info.get(k, None)
            if member is None:
                member, k = flat_type_info.alt.get(k, (None, k))
                if member is None:
                    continue

            member_attrs = self.get_cls_attrs(member)

            if member_attrs.exc:
                continue

            mo = member_attrs.max_occurs
            if mo > 1:
                subinst = getattr(inst, k, None)
                if subinst is None:
                    subinst = []

                for a in v:
                    subinst.append(
                            self._from_dict_value(ctx, k, member, a, validator))

            else:
                subinst = self._from_dict_value(ctx, k, member, v, validator)

            inst._safe_set(k, subinst, member, member_attrs)

            frequencies[k] += 1

        attrs = self.get_cls_attrs(cls)
        if validator is self.SOFT_VALIDATION and attrs.validate_freq:
            self._check_freq_dict(cls, frequencies, flat_type_info)

        return inst

    def _object_to_doc(self, cls, inst, tags=None):
        if inst is None:
            return None

        if tags is None:
            tags = set()

        retval = None

        if isinstance(inst, Fault):
            retval = None
            inst_id = id(inst)
            if not (inst_id in tags):
                retval = self._fault_to_doc(inst, cls)
                tags.add(inst_id)
            return retval

        cls_attrs = self.get_cls_attrs(cls)
        if cls_attrs.exc:
            return

        cls_orig = None
        if cls_attrs.out_type is not None:
            cls_orig = cls
            cls = cls_attrs.out_type
            # remember to do this if cls_attrs are needed below
            # (currently cls_attrs is not used so we don't do this)
            # cls_attrs = self.get_cls_attrs(cls)

        elif cls_attrs.type is not None:
            cls_orig = cls
            cls = cls_attrs.type
            # remember to do this if cls_attrs are needed below
            # (currently cls_attrs is not used so we don't do this)
            # cls_attrs = self.get_cls_attrs(cls)

        if self.ignore_wrappers:
            ti = getattr(cls, '_type_info', {})

            while cls.Attributes._wrapper and len(ti) == 1:
                # Wrappers are auto-generated objects that have exactly one
                # child type.
                key, = ti.keys()
                if not issubclass(cls, Array):
                    inst = getattr(inst, key, None)
                cls, = ti.values()
                ti = getattr(cls, '_type_info', {})

        # transform the results into a dict:
        if cls.Attributes.max_occurs > 1:
            if inst is not None:
                retval = []

                for subinst in inst:
                    if id(subinst) in tags:
                        # even when there is ONE already-serialized instance,
                        # we throw the whole thing away.
                        logger.debug("Throwing the whole array away because "
                                                        "found %d", id(subinst))

                        # this is DANGEROUS
                        #logger.debug("Said array: %r", inst)

                        return None

                    retval.append(self._to_dict_value(cls, subinst, tags,
                                                      cls_orig=cls_orig or cls))

        else:
            retval = self._to_dict_value(cls, inst, tags,
                                                       cls_orig=cls_orig or cls)

        return retval

    def _get_member_pairs(self, cls, inst, tags):
        old_len = len(tags)
        tags = tags | {id(inst)}
        assert len(tags) > old_len, ("Offending instance: %r" % inst)

        for k, v in self.sort_fields(cls):
            subattr = self.get_cls_attrs(v)

            if subattr.exc:
                continue

            try:
                subinst = getattr(inst, k, None)

            # to guard against e.g. sqlalchemy throwing NoSuchColumnError
            except Exception as e:
                logger.error("Error getting %r: %r" % (k, e))
                subinst = None

            if subinst is None:
                subinst = subattr.default
            else:
                if id(subinst) in tags:
                    continue

            logger.debug("%s%r type is %r", "  " * len(tags), k, v)

            val = self._object_to_doc(v, subinst, tags)
            min_o = subattr.min_occurs

            complex_as = self.get_complex_as(subattr)
            if val is not None or min_o > 0 or complex_as is list:
                sub_name = subattr.sub_name
                if sub_name is None:
                    sub_name = k

                yield (sub_name, val)

    def _to_dict_value(self, cls, inst, tags, cls_orig=None):
        if cls_orig is None:
            cls_orig = cls
        cls, switched = self.get_polymorphic_target(cls, inst)
        cls_attrs = self.get_cls_attrs(cls)

        inst = self._sanitize(cls_attrs, inst)

        if issubclass(cls_orig, File):
            cls_orig_attrs = self.get_cls_attrs(cls_orig)
            if not isinstance(inst, cls_orig_attrs.type):
                return self.to_serstr(cls_orig, inst, self.binary_encoding)

            retval = self._complex_to_doc(cls_orig_attrs.type, inst, tags)
            complex_as = self.get_complex_as(cls_orig_attrs)

            if complex_as is dict and not self.ignore_wrappers:
                retval = next(iter(retval.values()))

            return retval

        if issubclass(cls, (Any, AnyDict)):
            return inst

        if issubclass(cls, Array):
            st, = cls._type_info.values()
            return self._object_to_doc(st, inst, tags)

        if issubclass(cls, ComplexModelBase):
            return self._complex_to_doc(cls, inst, tags)

        if issubclass(cls, (ByteArray, Uuid)):
            return self.to_serstr(cls, inst, self.binary_encoding)

        return self.to_serstr(cls, inst)

    def _complex_to_doc(self, cls, inst, tags):
        cls_attrs = self.get_cls_attrs(cls)
        sf = cls_attrs.simple_field
        if sf is not None:
            # we want this to throw when sf does not exist
            subcls = cls.get_flat_type_info(cls)[sf]

            subinst = getattr(inst, sf, None)

            logger.debug("Render complex object %s to the value %r of its "
                         "field '%s'", cls.get_type_name(), subinst, sf)

            return self.to_unicode(subcls, subinst)

        cls_attr = self.get_cls_attrs(cls)
        complex_as = self.get_complex_as(cls_attr)
        if complex_as is list or \
                         getattr(cls.Attributes, 'serialize_as', False) is list:
            return list(self._complex_to_list(cls, inst, tags))
        return self._complex_to_dict(cls, inst, tags)

    def _complex_to_dict(self, cls, inst, tags):
        inst = cls.get_serialization_instance(inst)
        cls_attr = self.get_cls_attrs(cls)
        complex_as = self.get_complex_as(cls_attr)

        if self.key_encoding is None:
            d = complex_as(self._get_member_pairs(cls, inst, tags))

            if (self.ignore_wrappers or cls_attr.not_wrapped) \
                                                 and not bool(cls_attr.wrapper):
                return d

            else:
                if isinstance(cls_attr.wrapper,
                                              (six.text_type, six.binary_type)):
                    return {cls_attr.wrapper: d}
                else:
                    return {cls.get_type_name(): d}
        else:
            d = complex_as( (k.encode(self.key_encoding), v) for k, v in
                                       self._get_member_pairs(cls, inst, tags) )

            if (self.ignore_wrappers or cls_attr.not_wrapped) \
                                                 and not bool(cls_attr.wrapper):
                return d

            else:
                if isinstance(cls_attr.wrapper, six.text_type):
                    return {cls_attr.wrapper.encode(self.key_encoding): d}
                elif isinstance(cls_attr.wrapper, six.binary_type):
                    return {cls_attr.wrapper: d}
                else:
                    return {cls.get_type_name().encode(self.key_encoding): d}

    def _complex_to_list(self, cls, inst, tags):
        inst = cls.get_serialization_instance(inst)

        for k, v in self._get_member_pairs(cls, inst, tags):
            yield v



#!/usr/bin/env python
# encoding: utf8
#
# Copyright © Burak Arslan <burak at arskom dot com dot tr>,
#             Arskom Ltd. http://www.arskom.com.tr
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    1. Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#    2. Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#    3. Neither the name of the owner nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#


"""
This is a blocking example running in a multi-threaded twisted setup.

This is a way of weakly integrating with the twisted framework -- every request
still runs in its own thread. This way, you can still use other features of
twisted and not have to rewrite your otherwise synchronous code.

    $ time curl -s "http://localhost:9757/block?seconds=10" > /dev/null & \
      time curl -s "http://localhost:9757/block?seconds=10" > /dev/null &
    [1] 27537
    [2] 27538

    real    0m10.031s
    user    0m0.008s
    sys     0m0.007s

    real    0m10.038s
    user    0m0.006s
    sys     0m0.006s
"""

import sys
import time
import logging

from twisted.internet import reactor
from twisted.web.server import Site
from twisted.web.wsgi import WSGIResource
from twisted.python import log

from spyne import Application, rpc, Service, Integer
from spyne.server.wsgi import WsgiApplication
from spyne.protocol.http import HttpRpc


HOST = '0.0.0.0'
PORT = 9757


class SomeService(Service):
    @rpc(Integer, _returns=Integer)
    def block(ctx, seconds):
        """Blocks the current thread for given number of seconds."""
        time.sleep(seconds)
        return seconds


def initialize(services, tns='spyne.examples.twisted.resource'):
    logging.basicConfig(level=logging.DEBUG)
    logging.getLogger('spyne.protocol.xml').setLevel(logging.DEBUG)

    observer = log.PythonLoggingObserver('twisted')
    log.startLoggingWithObserver(observer.emit, setStdout=False)

    application = Application(services, 'spyne.examples.twisted.hello',
                                in_protocol=HttpRpc(), out_protocol=HttpRpc())

    return application


if __name__ == '__main__':
    application = initialize([SomeService])
    wsgi_application = WsgiApplication(application)
    resource = WSGIResource(reactor, reactor, wsgi_application)
    site = Site(resource)

    reactor.listenTCP(PORT, site, interface=HOST)

    logging.info('listening on: %s:%d' % (HOST,PORT))
    logging.info('wsdl is at: http://%s:%d/?wsdl' % (HOST, PORT))

    sys.exit(reactor.run())



#!/usr/bin/env python

# This can be used to debug invalid Xml Schema documents.

import sys

from lxml import etree

if len(sys.argv) != 2:
    print "Usage: %s <path_to_xsd_file>" % sys.argv[0]
    sys.exit(1)

f = open(sys.argv[1])

etree.XMLSchema(etree.parse(f))




#
# spyne - Copyright (C) Spyne contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

"""The cooperative multitasking module. It includes the coroutine stuff.

This could have been named just coroutine.py if it wasn't for the coroutine
decorator.
"""


import logging
logger = logging.getLogger(__name__)

from itertools import chain
from inspect import isgeneratorfunction


class Break(Exception):
    """Raised for breaking out of infinite loops inside coroutines."""
    pass


def coroutine(func):
    assert isgeneratorfunction(func)

    def start(*args, **kwargs):
        try:
            ret = func(*args, **kwargs)
        except TypeError as e:
            logger.error("Function %r at %s:%d got error %r", func.func_name,
                         func.__module__, func.__code__.co_firstlineno, e)
            raise

        try:
            next(ret)

        except StopIteration:
            return None

        except Exception as e:
            if not hasattr(e, 'logged'):
                logger.error("Exception in coroutine")
                logger.exception(e)
                try:
                    e.logged = True
                except:
                    pass

            raise

        return ret

    return start


def keepfirst(func):
    assert isgeneratorfunction(func)

    def start(*args, **kwargs):
        try:
            ret = func(*args, **kwargs)
        except TypeError as e:
            logger.error("Function %r at %s:%d got error %r", func.func_name,
                         func.__module__, func.__code__.co_firstlineno, e)
            raise

        try:
            first = next(ret)

        except StopIteration:
            return None

        except Exception as e:
            if not hasattr(e, 'logged'):
                logger.error("Exception in coroutine")
                logger.exception(e)
                try:
                    e.logged = True
                except:
                    pass

            raise

        return chain((first,), ret)

    return start



#!/usr/bin/env python
# encoding: utf8
#
# Copyright © Burak Arslan <burak at arskom dot com dot tr>,
#             Arskom Ltd. http://www.arskom.com.tr
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    1. Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#    2. Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#    3. Neither the name of the owner nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

"""
This is a simple HelloWorld example to show the basics of writing a Http api
using Spyne's push utilities.
"""


import logging

from spyne.application import Application
from spyne.decorator import rpc
from spyne.protocol.html import HtmlColumnTable
from spyne.protocol.http import HttpRpc
from spyne.service import Service
from spyne.model.complex import Iterable
from spyne.model.primitive import UnsignedInteger
from spyne.model.primitive import String
from spyne.server.wsgi import WsgiApplication


class HelloWorldService(Service):
    @rpc(String, UnsignedInteger, _returns=Iterable(String))
    def say_hello(ctx, name, times):
        def cb(ret):
            for i in range(times):
                ret.append('Hello, %s' % name)

        return Iterable.Push(cb)

if __name__=='__main__':
    from lxml.builder import E
    logging.basicConfig(level=logging.DEBUG)

    cloth = E.html(E.body(
        E.style("""
            td,th {
                border-left: 1px solid #ccc;
                border-right: 1px solid #ccc;
                border-bottom: 1px solid;
                margin: 0;
            }""", type="text/css"
        ),
        spyne="",
    ))

    application = Application([HelloWorldService], 'spyne.examples.hello.http',
        in_protocol=HttpRpc(validator='soft'),
        out_protocol=HtmlColumnTable(cloth=cloth),
    )

    wsgi_application = WsgiApplication(application)

    from wsgiref.simple_server import make_server
    server = make_server('127.0.0.1', 8000, wsgi_application)

    logging.info("listening to http://127.0.0.1:8000")
    logging.info("wsdl is at: http://localhost:8000/?wsdl")

    server.serve_forever()



#!/usr/bin/env python
#
# spyne - Copyright (C) Spyne contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import logging
logging.basicConfig(level=logging.DEBUG)

import inspect
import unittest
import sqlalchemy

from pprint import pprint

from sqlalchemy import create_engine
from sqlalchemy import MetaData
from sqlalchemy import Column
from sqlalchemy import Table
from sqlalchemy.exc import IntegrityError

from sqlalchemy.orm import mapper
from sqlalchemy.orm import sessionmaker

from spyne import M, Any, Double

from spyne.model import XmlAttribute, File, XmlData, ComplexModel, Array, \
    Integer32, Unicode, Integer, Enum, TTableModel, DateTime, Boolean

from spyne.model.binary import HybridFileStore
from spyne.model.complex import xml
from spyne.model.complex import table

from spyne.store.relational import get_pk_columns
from spyne.store.relational.document import PGJsonB, PGJson, PGFileJson, \
    PGObjectJson

TableModel = TTableModel()


class TestSqlAlchemyTypeMappings(unittest.TestCase):
    def test_init(self):
        fn = inspect.stack()[0][3]
        from sqlalchemy.inspection import inspect as sqla_inspect

        class SomeClass1(TableModel):
            __tablename__ = "%s_%d" % (fn, 1)
            i = Integer32(pk=True)
            e = Unicode(32)

        from spyne.util.dictdoc import get_dict_as_object
        inst = get_dict_as_object(dict(i=4), SomeClass1)
        assert not sqla_inspect(inst).attrs.e.history.has_changes()

    def test_bool(self):
        fn = inspect.stack()[0][3]

        class SomeClass1(TableModel):
            __tablename__ = "%s_%d" % (fn, 1)
            i = Integer32(pk=True)
            b = Boolean

        assert isinstance(SomeClass1.Attributes.sqla_table.c.b.type,
                                                             sqlalchemy.Boolean)

        class SomeClass2(TableModel):
            __tablename__ = "%s_%d" % (fn, 2)
            i = Integer32(pk=True)
            b = Boolean(store_as=int)

        assert isinstance(SomeClass2.Attributes.sqla_table.c.b.type,
                                                        sqlalchemy.SmallInteger)

    def test_jsonb(self):
        fn = inspect.stack()[0][3]

        class SomeClass1(TableModel):
            __tablename__ = "%s_%d" % (fn, 1)
            i = Integer32(pk=True)
            a = Any(store_as='json')

        assert isinstance(SomeClass1.Attributes.sqla_table.c.a.type, PGJson)

        class SomeClass2(TableModel):
            __tablename__ = "%s_%d" % (fn, 2)
            i = Integer32(pk=True)
            a = Any(store_as='jsonb')

        assert isinstance(SomeClass2.Attributes.sqla_table.c.a.type, PGJsonB)

        class SomeClass3(TableModel):
            __tablename__ = "%s_%d" % (fn, 3)
            i = Integer32(pk=True)
            a = File(store_as=HybridFileStore("path", db_format='jsonb'))

        assert isinstance(SomeClass3.Attributes.sqla_table.c.a.type, PGFileJson)
        assert SomeClass3.Attributes.sqla_table.c.a.type.dbt == 'jsonb'

    def test_obj_json(self):
        fn = inspect.stack()[0][3]

        class SomeClass(ComplexModel):
            s = Unicode
            d = Double

        class SomeClass1(TableModel):
            __tablename__ = "%s_%d" % (fn, 1)
            _type_info = [
                ('i', Integer32(pk=True)),
                ('a', Array(SomeClass, store_as='json')),
            ]

        assert isinstance(SomeClass1.Attributes.sqla_table.c.a.type,
                                                                   PGObjectJson)

        class SomeClass2(TableModel):
            __tablename__ = "%s_%d" % (fn, 2)
            i = Integer32(pk=True)
            a = SomeClass.customize(store_as='json')

        assert isinstance(SomeClass2.Attributes.sqla_table.c.a.type,
                                                                   PGObjectJson)


class TestSqlAlchemySchema(unittest.TestCase):
    def setUp(self):
        logging.getLogger('sqlalchemy').setLevel(logging.DEBUG)

        self.engine = create_engine('sqlite:///:memory:')
        self.session = sessionmaker(bind=self.engine)()
        self.metadata = TableModel.Attributes.sqla_metadata = MetaData()
        self.metadata.bind = self.engine
        logging.info('Testing against sqlalchemy-%s', sqlalchemy.__version__)

    def test_obj_json_dirty(self):
        fn = inspect.stack()[0][3]

        class SomeClass(ComplexModel):
            s = Unicode
            d = Double

        class SomeClass1(TableModel):
            __tablename__ = "%s_%d" % (fn, 1)
            _type_info = [
                ('i', Integer32(pk=True)),
                ('a', SomeClass.store_as('jsonb')),
            ]

        self.metadata.create_all()

        sc1 = SomeClass1(i=5, a=SomeClass(s="s", d=42.0))
        self.session.add(sc1)
        self.session.commit()

        from sqlalchemy.orm.attributes import flag_modified

        # TODO: maybe do the flag_modified() on setitem?
        sc1.a.s = "ss"
        flag_modified(sc1, 'a')

        assert sc1 in self.session.dirty

        self.session.commit()
        assert sc1.a.s == "ss"

        # not implemented
        #sc1.a[0].s = "sss"
        #flag_modified(sc1.a[0], 's')
        #assert sc1.a[0] in self.session.dirty

    def test_schema(self):
        class SomeClass(TableModel):
            __tablename__ = 'some_class'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True, autoincrement=False)
            s = Unicode(64, unique=True)
            i = Integer32(64, index=True)

        t = SomeClass.__table__
        self.metadata.create_all()  # not needed, just nice to see.

        assert t.c.id.primary_key == True
        assert t.c.id.autoincrement == False
        indexes = list(t.indexes)
        indexes.sort(key=lambda idx: idx.name)
        for idx in indexes:
            assert 'i' in idx.columns or 's' in idx.columns
            if 's' in idx.columns:
                assert idx.unique

    def test_colname_simple(self):
        class SomeClass(TableModel):
            __tablename__ = 'some_class'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True, autoincrement=False)
            s = Unicode(64, sqla_column_args=dict(name='ss'))

        t = SomeClass.__table__
        self.metadata.create_all()  # not needed, just nice to see.

        assert 'ss' in t.c

    def test_colname_complex_table(self):
        class SomeOtherClass(TableModel):
            __tablename__ = 'some_other_class'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True)
            s = Unicode(64)

        class SomeClass(TableModel):
            __tablename__ = 'some_class'
            __table_args__ = (
                {"sqlite_autoincrement": True},
            )

            id = Integer32(primary_key=True)
            o = SomeOtherClass.customize(store_as='table',
                                               sqla_column_args=dict(name='oo'))

        t = SomeClass.__table__
        self.metadata.create_all()  # not needed, just nice to see.

        assert 'oo_id' in t.c

    def test_colname_complex_json(self):
        class SomeOtherClass(TableModel):
            __tablename__ = 'some_other_class'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True)
            s = Unicode(64)

        class SomeClass(TableModel):
            __tablename__ = 'some_class'
            __table_args__ = (
                {"sqlite_autoincrement": True},
            )

            id = Integer32(primary_key=True)
            o = SomeOtherClass.customize(store_as='json',
                                               sqla_column_args=dict(name='oo'))

        t = SomeClass.__table__
        self.metadata.create_all()  # not needed, just nice to see.

        assert 'oo' in t.c

    def test_nested_sql(self):
        class SomeOtherClass(TableModel):
            __tablename__ = 'some_other_class'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True)
            s = Unicode(64)

        class SomeClass(TableModel):
            __tablename__ = 'some_class'
            __table_args__ = (
                {"sqlite_autoincrement": True},
            )

            id = Integer32(primary_key=True)
            o = SomeOtherClass.customize(store_as='table')

        self.metadata.create_all()

        soc = SomeOtherClass(s='ehe')
        sc = SomeClass(o=soc)

        self.session.add(sc)
        self.session.commit()
        self.session.close()

        sc_db = self.session.query(SomeClass).get(1)
        print(sc_db)
        assert sc_db.o.s == 'ehe'
        assert sc_db.o_id == 1

        sc_db.o = None
        self.session.commit()
        self.session.close()

        sc_db = self.session.query(SomeClass).get(1)
        assert sc_db.o == None
        assert sc_db.o_id == None

    def test_nested_sql_array_as_table(self):
        class SomeOtherClass(TableModel):
            __tablename__ = 'some_other_class'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True)
            s = Unicode(64)

        class SomeClass(TableModel):
            __tablename__ = 'some_class'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True)
            others = Array(SomeOtherClass, store_as='table')

        self.metadata.create_all()

        soc1 = SomeOtherClass(s='ehe1')
        soc2 = SomeOtherClass(s='ehe2')
        sc = SomeClass(others=[soc1, soc2])

        self.session.add(sc)
        self.session.commit()
        self.session.close()

        sc_db = self.session.query(SomeClass).get(1)

        assert sc_db.others[0].s == 'ehe1'
        assert sc_db.others[1].s == 'ehe2'

        self.session.close()

    def test_nested_sql_array_as_multi_table(self):
        class SomeOtherClass(TableModel):
            __tablename__ = 'some_other_class'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True)
            s = Unicode(64)

        class SomeClass(TableModel):
            __tablename__ = 'some_class'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True)
            others = Array(SomeOtherClass, store_as=table(multi=True))

        self.metadata.create_all()

        soc1 = SomeOtherClass(s='ehe1')
        soc2 = SomeOtherClass(s='ehe2')
        sc = SomeClass(others=[soc1, soc2])

        self.session.add(sc)
        self.session.commit()
        self.session.close()

        sc_db = self.session.query(SomeClass).get(1)

        assert sc_db.others[0].s == 'ehe1'
        assert sc_db.others[1].s == 'ehe2'

        self.session.close()

    def test_nested_sql_array_as_multi_table_with_backref(self):
        class SomeOtherClass(TableModel):
            __tablename__ = 'some_other_class'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True)
            s = Unicode(64)

        class SomeClass(TableModel):
            __tablename__ = 'some_class'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True)
            others = Array(SomeOtherClass,
                             store_as=table(multi=True, backref='some_classes'))

        self.metadata.create_all()

        soc1 = SomeOtherClass(s='ehe1')
        soc2 = SomeOtherClass(s='ehe2')
        sc = SomeClass(others=[soc1, soc2])

        self.session.add(sc)
        self.session.commit()
        self.session.close()

        soc_db = self.session.query(SomeOtherClass).all()

        assert soc_db[0].some_classes[0].id == 1
        assert soc_db[1].some_classes[0].id == 1

        self.session.close()

    def test_nested_sql_array_as_xml(self):
        class SomeOtherClass(ComplexModel):
            id = Integer32
            s = Unicode(64)

        class SomeClass(TableModel):
            __tablename__ = 'some_class'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True)
            others = Array(SomeOtherClass, store_as='xml')

        self.metadata.create_all()

        soc1 = SomeOtherClass(s='ehe1')
        soc2 = SomeOtherClass(s='ehe2')
        sc = SomeClass(others=[soc1, soc2])

        self.session.add(sc)
        self.session.commit()
        self.session.close()

        sc_db = self.session.query(SomeClass).get(1)

        assert sc_db.others[0].s == 'ehe1'
        assert sc_db.others[1].s == 'ehe2'

        self.session.close()

    def test_nested_sql_array_as_xml_no_ns(self):
        class SomeOtherClass(ComplexModel):
            id = Integer32
            s = Unicode(64)

        class SomeClass(TableModel):
            __tablename__ = 'some_class'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True)
            others = Array(SomeOtherClass, store_as=xml(no_ns=True))

        self.metadata.create_all()

        soc1 = SomeOtherClass(s='ehe1')
        soc2 = SomeOtherClass(s='ehe2')
        sc = SomeClass(others=[soc1, soc2])

        self.session.add(sc)
        self.session.commit()
        self.session.close()

        sc_xml = self.session.connection() \
                     .execute("select others from some_class") .fetchall()[0][0]

        from lxml import etree
        assert etree.fromstring(sc_xml).tag == 'SomeOtherClassArray'

        self.session.close()

    def test_inheritance(self):
        class SomeOtherClass(TableModel):
            __tablename__ = 'some_other_class'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True)
            s = Unicode(64)

        class SomeClass(SomeOtherClass):
            numbers = Array(Integer32).store_as(xml(no_ns=True, root_tag='a'))

        self.metadata.create_all()

        sc = SomeClass(id=5, s='s', numbers=[1, 2, 3, 4])

        self.session.add(sc)
        self.session.commit()
        self.session.close()

        sc_db = self.session.query(SomeClass).get(5)
        assert sc_db.numbers == [1, 2, 3, 4]
        self.session.close()

        sc_db = self.session.query(SomeOtherClass).get(5)
        assert sc_db.id == 5
        try:
            sc_db.numbers
        except AttributeError:
            pass
        else:
            raise Exception("must fail")

        self.session.close()

    def test_inheritance_with_complex_fields(self):
        class Foo(TableModel):
            __tablename__ = 'foo'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True)
            s = Unicode(64)

        class Bar(TableModel):
            __tablename__ = 'bar'
            __table_args__ = {"sqlite_autoincrement": True}
            __mapper_args__ = {
                'polymorphic_on': 'type',
                'polymorphic_identity': 'bar',
                'with_polymorphic': '*',
            }

            id = Integer32(primary_key=True)
            s = Unicode(64)
            type = Unicode(6)
            foos = Array(Foo).store_as('table')

        class SubBar(Bar):
            __mapper_args__ = {
                'polymorphic_identity': 'subbar',
            }
            i = Integer32

        sqlalchemy.orm.configure_mappers()

        mapper_subbar = SubBar.Attributes.sqla_mapper
        mapper_bar = Bar.Attributes.sqla_mapper
        assert not mapper_subbar.concrete

        for inheriting in mapper_subbar.iterate_to_root():
            if inheriting is not mapper_subbar \
                    and not (mapper_bar.relationships['foos'] is
                                           mapper_subbar.relationships['foos']):
                raise Exception("Thou shalt stop children relationships "
                                           "from overriding the ones in parent")

    def test_mixins_with_complex_fields(self):
        class Foo(TableModel):
            __tablename__ = 'foo'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True)
            s = Unicode(64)

        class Bar(TableModel):
            __tablename__ = 'bar'
            __table_args__ = {"sqlite_autoincrement": True}
            __mixin__ = True
            __mapper_args__ = {
                'polymorphic_on': 'type',
                'polymorphic_identity': 'bar',
                'with_polymorphic': '*',
            }

            id = Integer32(primary_key=True)
            s = Unicode(64)
            type = Unicode(6)
            foos = Array(Foo).store_as('table')

        class SubBar(Bar):
            __mapper_args__ = {
                'polymorphic_identity': 'subbar',
            }
            i = Integer32

        sqlalchemy.orm.configure_mappers()

        mapper_subbar = SubBar.Attributes.sqla_mapper
        mapper_bar = Bar.Attributes.sqla_mapper
        assert not mapper_subbar.concrete

        for inheriting in mapper_subbar.iterate_to_root():
            if inheriting is not mapper_subbar \
                    and not (mapper_bar.relationships['foos'] is
                                           mapper_subbar.relationships['foos']):
                raise Exception("Thou shalt stop children relationships "
                                           "from overriding the ones in parent")

    def test_sqlalchemy_inheritance(self):
        # no spyne code is involved here.
        # this is just to test test the sqlalchemy behavior that we rely on.

        class Employee(object):
            def __init__(self, name):
                self.name = name

            def __repr__(self):
                return self.__class__.__name__ + " " + self.name

        class Manager(Employee):
            def __init__(self, name, manager_data):
                self.name = name
                self.manager_data = manager_data

            def __repr__(self):
                return (
                    self.__class__.__name__ + " " +
                    self.name + " " + self.manager_data
                )

        class Engineer(Employee):
            def __init__(self, name, engineer_info):
                self.name = name
                self.engineer_info = engineer_info

            def __repr__(self):
                return (
                    self.__class__.__name__ + " " +
                    self.name + " " + self.engineer_info
                )

        employees_table = Table('employees', self.metadata,
            Column('employee_id', sqlalchemy.Integer, primary_key=True),
            Column('name', sqlalchemy.String(50)),
            Column('manager_data', sqlalchemy.String(50)),
            Column('engineer_info', sqlalchemy.String(50)),
            Column('type', sqlalchemy.String(20), nullable=False),
        )

        employee_mapper = mapper(Employee, employees_table,
                                        polymorphic_on=employees_table.c.type,
                                                polymorphic_identity='employee')

        manager_mapper = mapper(Manager, inherits=employee_mapper,
                                                polymorphic_identity='manager')

        engineer_mapper = mapper(Engineer, inherits=employee_mapper,
                                                polymorphic_identity='engineer')

        self.metadata.create_all()

        manager = Manager('name', 'data')
        self.session.add(manager)
        self.session.commit()
        self.session.close()

        assert self.session.query(Employee).with_polymorphic('*') \
                   .filter_by(employee_id=1) \
                   .one().type == 'manager'

    def test_inheritance_polymorphic_with_non_nullables_in_subclasses(self):
        class SomeOtherClass(TableModel):
            __tablename__ = 'some_other_class'
            __table_args__ = {"sqlite_autoincrement": True}
            __mapper_args__ = {'polymorphic_on': 't', 'polymorphic_identity': 1}

            id = Integer32(primary_key=True)
            t = Integer32(nillable=False)
            s = Unicode(64, nillable=False)

        class SomeClass(SomeOtherClass):
            __mapper_args__ = (
                (),
                {'polymorphic_identity': 2},
            )

            i = Integer(nillable=False)

        self.metadata.create_all()

        assert SomeOtherClass.__table__.c.s.nullable == False

        # this should be nullable to let other classes be added.
        # spyne still checks this constraint when doing input validation.
        # spyne should generate a constraint to check this at database level as
        # well.
        assert SomeOtherClass.__table__.c.i.nullable == True

        soc = SomeOtherClass(s='s')
        self.session.add(soc)
        self.session.commit()
        soc_id = soc.id

        try:
            sc = SomeClass(i=5)
            self.session.add(sc)
            self.session.commit()
        except IntegrityError:
            self.session.rollback()
        else:
            raise Exception("Must fail with IntegrityError.")

        sc2 = SomeClass(s='s')  # this won't fail. should it?
        self.session.add(sc2)
        self.session.commit()

        self.session.expunge_all()

        assert self.session.query(SomeOtherClass).with_polymorphic('*') \
                   .filter_by(id=soc_id).one().t == 1

        self.session.close()

    def test_inheritance_polymorphic(self):
        class SomeOtherClass(TableModel):
            __tablename__ = 'some_class'
            __table_args__ = {"sqlite_autoincrement": True}
            __mapper_args__ = {'polymorphic_on': 't', 'polymorphic_identity': 1}

            id = Integer32(primary_key=True)
            s = Unicode(64)
            t = Integer32(nillable=False)

        class SomeClass(SomeOtherClass):
            __mapper_args__ = {'polymorphic_identity': 2}
            numbers = Array(Integer32).store_as(xml(no_ns=True, root_tag='a'))

        self.metadata.create_all()

        sc = SomeClass(id=5, s='s', numbers=[1, 2, 3, 4])

        self.session.add(sc)
        self.session.commit()
        self.session.close()

        assert self.session.query(SomeOtherClass).with_polymorphic('*') \
            .filter_by(id=5).one().t == 2
        self.session.close()

    def test_nested_sql_array_as_json(self):
        class SomeOtherClass(ComplexModel):
            id = Integer32
            s = Unicode(64)

        class SomeClass(TableModel):
            __tablename__ = 'some_class'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True)
            others = Array(SomeOtherClass, store_as='json')

        self.metadata.create_all()

        soc1 = SomeOtherClass(s='ehe1')
        soc2 = SomeOtherClass(s='ehe2')
        sc = SomeClass(others=[soc1, soc2])

        self.session.add(sc)
        self.session.commit()
        self.session.close()

        sc_db = self.session.query(SomeClass).get(1)

        assert sc_db.others[0].s == 'ehe1'
        assert sc_db.others[1].s == 'ehe2'

        self.session.close()

    def test_modifiers(self):
        class SomeClass(TableModel):
            __tablename__ = 'some_class'
            __table_args__ = {"sqlite_autoincrement": True}

            i = XmlAttribute(Integer32(pk=True))
            s = XmlData(Unicode(64))

        self.metadata.create_all()
        self.session.add(SomeClass(s='s'))
        self.session.commit()
        self.session.expunge_all()

        ret = self.session.query(SomeClass).get(1)
        assert ret.i == 1  # redundant
        assert ret.s == 's'

    def test_default_ctor(self):
        class SomeOtherClass(ComplexModel):
            id = Integer32
            s = Unicode(64)

        class SomeClass(TableModel):
            __tablename__ = 'some_class'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True)
            others = Array(SomeOtherClass, store_as='json')
            f = Unicode(32, default='uuu')

        self.metadata.create_all()
        self.session.add(SomeClass())
        self.session.commit()
        self.session.expunge_all()

        assert self.session.query(SomeClass).get(1).f == 'uuu'

    def test_default_value(self):
        class SomeClass(TableModel):
            __tablename__ = 'some_class'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True)
            f = Unicode(32, db_default=u'uuu')

        self.metadata.create_all()
        val = SomeClass()
        assert val.f is None

        self.session.add(val)
        self.session.commit()

        self.session.expunge_all()

        assert self.session.query(SomeClass).get(1).f == u'uuu'

    def test_default_ctor_with_sql_relationship(self):
        class SomeOtherClass(TableModel):
            __tablename__ = 'some_other_class'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True)
            s = Unicode(64)

        class SomeClass(TableModel):
            __tablename__ = 'some_class'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True)
            o = SomeOtherClass.customize(store_as='table')

        self.metadata.create_all()
        self.session.add(SomeClass())
        self.session.commit()

    def test_store_as_index(self):
        class SomeOtherClass(TableModel):
            __tablename__ = 'some_other_class'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True)
            s = Unicode(64)

        class SomeClass(TableModel):
            __tablename__ = 'some_class'
            __table_args__ = {"sqlite_autoincrement": True}

            id = Integer32(primary_key=True)
            o = SomeOtherClass.customize(store_as='table', index='btree')

        self.metadata.create_all()
        idx, = SomeClass.__table__.indexes
        assert 'o_id' in idx.columns

    def test_scalar_collection(self):
        class SomeClass(TableModel):
            __tablename__ = 'some_class'

            id = Integer32(primary_key=True)
            values = Array(Unicode).store_as('table')

        self.metadata.create_all()

        self.session.add(SomeClass(id=1, values=['a', 'b', 'c']))
        self.session.commit()
        sc = self.session.query(SomeClass).get(1)
        assert sc.values == ['a', 'b', 'c']
        del sc

        sc = self.session.query(SomeClass).get(1)
        sc.values.append('d')
        self.session.commit()
        del sc
        sc = self.session.query(SomeClass).get(1)
        assert sc.values == ['a', 'b', 'c', 'd']

        sc = self.session.query(SomeClass).get(1)
        sc.values = sc.values[1:]
        self.session.commit()
        del sc
        sc = self.session.query(SomeClass).get(1)
        assert sc.values == ['b', 'c', 'd']

    def test_multiple_fk(self):
        class SomeChildClass(TableModel):
            __tablename__ = 'some_child_class'

            id = Integer32(primary_key=True)
            s = Unicode(64)
            i = Integer32

        class SomeClass(TableModel):
            __tablename__ = 'some_class'

            id = Integer32(primary_key=True)
            children = Array(SomeChildClass).store_as('table')
            mirror = SomeChildClass.store_as('table')

        self.metadata.create_all()

        children = [
            SomeChildClass(s='p', i=600),
            SomeChildClass(s='|', i=10),
            SomeChildClass(s='q', i=9),
        ]

        sc = SomeClass(children=children)
        self.session.add(sc)
        self.session.flush()
        sc.mirror = children[1]
        self.session.commit()
        del sc

        sc = self.session.query(SomeClass).get(1)
        assert ''.join([scc.s for scc in sc.children]) == 'p|q'
        assert sum([scc.i for scc in sc.children]) == 619

    def test_simple_fk(self):
        class SomeChildClass(TableModel):
            __tablename__ = 'some_child_class'

            id = Integer32(primary_key=True)
            s = Unicode(64)
            i = Integer32

        class SomeClass(TableModel):
            __tablename__ = 'some_class'

            id = Integer32(primary_key=True)
            child_id = Integer32(fk='some_child_class.id')

        foreign_keys = SomeClass.__table__.c['child_id'].foreign_keys
        assert len(foreign_keys) == 1
        fk, = foreign_keys
        assert fk._colspec == 'some_child_class.id'

    def test_multirel_single_table(self):
        class SomeChildClass(TableModel):
            __tablename__ = 'some_child_class'

            id = Integer32(primary_key=True)
            s = Unicode(64)

        class SomeOtherChildClass(TableModel):
            __tablename__ = 'some_other_child_class'

            id = Integer32(primary_key=True)
            i = Integer32

        class SomeClass(TableModel):
            __tablename__ = 'some_class'

            id = Integer32(primary_key=True)

            children = Array(SomeChildClass,
                store_as=table(
                    multi='children', lazy='joined',
                    left='parent_id', right='child_id',
                    fk_left_ondelete='cascade',
                    fk_right_ondelete='cascade',
                ),
            )

            other_children = Array(SomeOtherChildClass,
                store_as=table(
                    multi='children', lazy='joined',
                    left='parent_id', right='other_child_id',
                    fk_left_ondelete='cascade',
                    fk_right_ondelete='cascade',
                ),
            )

        t = SomeClass.Attributes.sqla_metadata.tables['children']

        fkp, = t.c.parent_id.foreign_keys
        assert fkp._colspec == 'some_class.id'

        fkc, = t.c.child_id.foreign_keys
        assert fkc._colspec == 'some_child_class.id'

        fkoc, = t.c.other_child_id.foreign_keys
        assert fkoc._colspec == 'some_other_child_class.id'

    def test_reflection(self):
        class SomeClass(TableModel):
            __tablename__ = 'some_class'

            id = Integer32(primary_key=True)
            s = Unicode(32)

        TableModel.Attributes.sqla_metadata.create_all()

        # create a new table model with empty metadata
        TM2 = TTableModel()
        TM2.Attributes.sqla_metadata.bind = self.engine

        # fill it with information from the db
        TM2.Attributes.sqla_metadata.reflect()

        # convert sqla info to spyne info
        class Reflected(TM2):
            __table__ = TM2.Attributes.sqla_metadata.tables['some_class']

        pprint(dict(Reflected._type_info).items())
        assert issubclass(Reflected._type_info['id'], Integer)

        # this looks at spyne attrs
        assert [k for k, v in get_pk_columns(Reflected)] == ['id']

        # this looks at sqla attrs
        assert [k for k, v in Reflected.get_primary_keys()] == ['id']

        assert issubclass(Reflected._type_info['s'], Unicode)
        assert Reflected._type_info['s'].Attributes.max_len == 32

    def _test_sqlalchemy_remapping(self):
        class SomeTable(TableModel):
            __tablename__ = 'some_table'
            id = Integer32(pk=True)
            i = Integer32
            s = Unicode(32)

        class SomeTableSubset(TableModel):
            __table__ = SomeTable.__table__

            id = Integer32(pk=True)  # sqla session doesn't work without pk
            i = Integer32

        class SomeTableOtherSubset(TableModel):
            __table__ = SomeTable.__table__
            _type_info = [(k, v) for k, v in SomeTable._type_info.items()
                                                            if k in ('id', 's')]

        self.session.add(SomeTable(id=1, i=2, s='s'))
        self.session.commit()

        st = self.session.query(SomeTable).get(1)
        sts = self.session.query(SomeTableSubset).get(1)
        stos = self.session.query(SomeTableOtherSubset).get(1)

        sts.i = 3
        sts.s = 'ss'  # will not be flushed to db
        self.session.commit()

        assert st.s == 's'
        assert stos.i == 3

    def test_file_storage(self):
        class C(TableModel):
            __tablename__ = "c"

            id = Integer32(pk=True)
            f = File(store_as=HybridFileStore('test_file_storage', 'json'))

        self.metadata.create_all()
        c = C(f=File.Value(name=u"name", type=u"type", data=[b"data"]))
        self.session.add(c)
        self.session.flush()
        self.session.commit()

        c = self.session.query(C).get(1)
        print(c)
        assert c.f.name == "name"
        assert c.f.type == "type"
        assert c.f.data[0][:] == b"data"

    def test_append_field_complex_existing_column(self):
        class C(TableModel):
            __tablename__ = "c"
            u = Unicode(pk=True)

        class D(TableModel):
            __tablename__ = "d"
            d = Integer32(pk=True)
            c = C.store_as('table')

        C.append_field('d', D.store_as('table'))
        assert C.Attributes.sqla_mapper.get_property('d').argument is D

    def test_append_field_complex_delayed(self):
        class C(TableModel):
            __tablename__ = "c"
            u = Unicode(pk=True)

        class D(C):
            i = Integer32

        C.append_field('d', DateTime)

        assert D.Attributes.sqla_mapper.has_property('d')

    def _test_append_field_complex_explicit_existing_column(self):
        # FIXME: Test something!

        class C(TableModel):
            __tablename__ = "c"
            id = Integer32(pk=True)

        # c already also produces c_id. this is undefined behaviour, one of them
        # gets ignored, whichever comes first.
        class D(TableModel):
            __tablename__ = "d"
            id = Integer32(pk=True)
            c = C.store_as('table')
            c_id = Integer32(15)

    def test_append_field_complex_circular_array(self):
        class C(TableModel):
            __tablename__ = "cc"
            id = Integer32(pk=True)

        class D(TableModel):
            __tablename__ = "dd"
            id = Integer32(pk=True)
            c = Array(C).customize(store_as=table(right='dd_id'))

        C.append_field('d', D.customize(store_as=table(left='dd_id')))
        self.metadata.create_all()

        c1, c2 = C(id=1), C(id=2)
        d = D(id=1, c=[c1, c2])
        self.session.add(d)
        self.session.commit()
        assert c1.d.id == 1

    def test_append_field_complex_new_column(self):
        class C(TableModel):
            __tablename__ = "c"
            u = Unicode(pk=True)

        class D(TableModel):
            __tablename__ = "d"
            id = Integer32(pk=True)

        C.append_field('d', D.store_as('table'))
        assert C.Attributes.sqla_mapper.get_property('d').argument is D
        assert isinstance(C.Attributes.sqla_table.c['d_id'].type,
                                                             sqlalchemy.Integer)

    def test_append_field_array(self):
        class C(TableModel):
            __tablename__ = "c"
            id = Integer32(pk=True)

        class D(TableModel):
            __tablename__ = "d"
            id = Integer32(pk=True)

        C.append_field('d', Array(D).store_as('table'))
        assert C.Attributes.sqla_mapper.get_property('d').argument is D
        print(repr(D.Attributes.sqla_table))
        assert isinstance(D.Attributes.sqla_table.c['c_id'].type,
                                                             sqlalchemy.Integer)

    def test_append_field_array_many(self):
        class C(TableModel):
            __tablename__ = "c"
            id = Integer32(pk=True)

        class D(TableModel):
            __tablename__ = "d"
            id = Integer32(pk=True)

        C.append_field('d', Array(D).store_as(table(multi='c_d')))
        assert C.Attributes.sqla_mapper.get_property('d').argument is D
        rel_table = C.Attributes.sqla_metadata.tables['c_d']
        assert 'c_id' in rel_table.c
        assert 'd_id' in rel_table.c

    def test_append_field_complex_cust(self):
        class C(TableModel):
            __tablename__ = "c"
            id = Integer32(pk=True)

        class D(TableModel):
            __tablename__ = "d"
            id = Integer32(pk=True)
            c = Array(C).store_as('table')

        C.append_field('d', D.customize(
            nullable=False,
            store_as=table(left='d_id'),
        ))
        assert C.__table__.c['d_id'].nullable == False

    def _test_append_field_cust(self):
        class C(TableModel):
            __tablename__ = "c"
            id = Integer32(pk=True)

        C2 = C.customize()

        C.append_field("s", Unicode)

        C()

        self.metadata.create_all()

        assert "s" in C2._type_info
        assert "s" in C2.Attributes.sqla_mapper.columns

        self.session.add(C2(s='foo'))
        self.session.commit()
        assert self.session.query(C).first().s == 'foo'

    def test_polymorphic_cust(self):
        class C(TableModel):
            __tablename__ = "c"
            __mapper_args__ = {
                'polymorphic_on': 't',
                'polymorphic_identity': 1,
            }

            id = Integer32(pk=True)
            t = M(Integer32)

        class D(C):
            __mapper_args__ = {
                'polymorphic_identity': 2,
            }
            d = Unicode

        D2 = D.customize()

        assert C().t == 1
        assert D().t == 2

        # That's the way SQLAlchemy works. Don't use customized classes in
        # anywhere other than interface definitions
        assert D2().t == None

    def test_base_append_simple(self):
        class B(TableModel):
            __tablename__ = 'b'
            __mapper_args__ = {
                'polymorphic_on': 't',
                'polymorphic_identity': 1,
            }

            id = Integer32(pk=True)
            t = M(Integer32)

        class C(B):
            __mapper_args__ = {
                'polymorphic_identity': 1,
            }
            s = Unicode

        B.append_field('i', Integer32)

        self.metadata.create_all()

        self.session.add(C(s="foo", i=42))
        self.session.commit()

        c = self.session.query(C).first()

        assert c.s == 'foo'
        assert c.i == 42
        assert c.t == 1

    def test_base_append_complex(self):
        class B(TableModel):
            __tablename__ = 'b'
            __mapper_args__ = {
                'polymorphic_on': 't',
                'polymorphic_identity': 1,
            }

            id = Integer32(pk=True)
            t = M(Integer32)

        class C(B):
            __mapper_args__ = {
                'polymorphic_identity': 1,
            }
            s = Unicode

        class D(TableModel):
            __tablename__ = 'd'
            id = Integer32(pk=True)
            i = M(Integer32)

        B.append_field('d', D.store_as('table'))

        self.metadata.create_all()

        self.session.add(C(d=D(i=42)))
        self.session.commit()

        c = self.session.query(C).first()

        assert c.d.i == 42


class TestSqlAlchemySchemaWithPostgresql(unittest.TestCase):
    def setUp(self):
        self.metadata = TableModel.Attributes.sqla_metadata = MetaData()

    def test_enum(self):
        table_name = "test_enum"

        enums = ('SUBSCRIBED', 'UNSUBSCRIBED', 'UNCONFIRMED')

        class SomeClass(TableModel):
            __tablename__ = table_name

            id = Integer32(primary_key=True)
            e = Enum(*enums, type_name='status_choices')

        t = self.metadata.tables[table_name]
        assert 'e' in t.c
        assert tuple(t.c.e.type.enums) == enums


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# encoding: utf8
#
# Copyright © Burak Arslan <burak at arskom dot com dot tr>,
#             Arskom Ltd. http://www.arskom.com.tr
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    1. Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#    2. Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#    3. Neither the name of the owner nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

"""
For testing different kind of arrays with different protocols.
"""


import logging

from spyne import Application, rpc, Service, Unicode, ComplexModel, Array

from spyne.protocol.xml import XmlDocument
from spyne.protocol.json import JsonDocument
from spyne.protocol.http import HttpRpc
from spyne.server.wsgi import WsgiApplication


class Permission(ComplexModel):
    __namespace__ = 'some_ns'

    application = Unicode
    feature = Unicode


v = [
    Permission(application='app', feature='f1'),
    Permission(application='app', feature='f2'),
]


class HelloWorldService(Service):
    @rpc(_returns=Array(Permission))
    def simple(ctx):
        return v

    @rpc(_returns=Permission.customize(max_occurs=float('inf')))
    def complex(ctx):
        return v


if __name__=='__main__':
    from wsgiref.simple_server import make_server
    logging.basicConfig(level=logging.DEBUG)

    application = Application([HelloWorldService], 'spyne.examples.hello.http',
          in_protocol=HttpRpc(validator='soft'),
          out_protocol=XmlDocument(),
      )

    wsgi_application = WsgiApplication(application)

    server = make_server('127.0.0.1', 8000, wsgi_application)

    logging.info("listening to http://127.0.0.1:8000")
    logging.info("wsdl is at: http://localhost:8000/?wsdl")

    server.serve_forever()




#
# spyne - Copyright (C) Spyne contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

import msgpack

from spyne import Application, Service, rpc
from spyne.model import Unicode
from spyne.protocol.msgpack import MessagePackDocument

from twisted.trial import  unittest


class TestMessagePackServer(unittest.TestCase):
     def gen_prot(self, app):
        from spyne.server.twisted.msgpack import TwistedMessagePackProtocol
        from twisted.test.proto_helpers import StringTransportWithDisconnection
        from spyne.server.msgpack import MessagePackServerBase

        prot = TwistedMessagePackProtocol(MessagePackServerBase(app))
        transport = StringTransportWithDisconnection()
        prot.makeConnection(transport)
        transport.protocol = prot

        return prot

     def test_roundtrip(self):
        v = "yaaay!"
        class SomeService(Service):
            @rpc(Unicode, _returns=Unicode)
            def yay(ctx, u):
                return u

        app = Application([SomeService], 'tns',
                                in_protocol=MessagePackDocument(),
                                out_protocol=MessagePackDocument())

        prot = self.gen_prot(app)
        request = msgpack.packb({'yay': [v]})
        prot.dataReceived(msgpack.packb([1, request]))
        val = prot.transport.value()
        print(repr(val))
        val = msgpack.unpackb(val)
        print(repr(val))

        self.assertEqual(val, [0, msgpack.packb(v)])

     def test_roundtrip_deferred(self):
        from twisted.internet import reactor
        from twisted.internet.task import deferLater

        v = "yaaay!"
        p_ctx = []
        class SomeService(Service):
            @rpc(Unicode, _returns=Unicode)
            def yay(ctx, u):
                def _cb():
                    return u
                p_ctx.append(ctx)
                return deferLater(reactor, 0.1, _cb)

        app = Application([SomeService], 'tns',
                                in_protocol=MessagePackDocument(),
                                out_protocol=MessagePackDocument())

        prot = self.gen_prot(app)
        request = msgpack.packb({'yay': [v]})
        def _ccb(_):
            val = prot.transport.value()
            print(repr(val))
            val = msgpack.unpackb(val)
            print(repr(val))

            self.assertEqual(val, [0, msgpack.packb(v)])

        prot.dataReceived(msgpack.packb([1, request]))

        return p_ctx[0].out_object[0].addCallback(_ccb)




#!/usr/bin/env python
# encoding: utf8
#
# Copyright © Burak Arslan <burak at arskom dot com dot tr>,
#             Arskom Ltd. http://www.arskom.com.tr
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    1. Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#    2. Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#    3. Neither the name of the owner nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

from __future__ import print_function


"""This shows the async push capabilities of Spyne using twisted in async mode.

In this example, user code runs directly in the reactor loop. Data is pushed to
the output slowly without blocking other requests.
"""


import logging
import sys

from twisted.internet import reactor
from twisted.internet.task import deferLater
from twisted.web.server import Site

from spyne import Application, rpc, Service, Iterable, Unicode, \
    UnsignedInteger
from spyne.protocol.http import HttpRpc
from spyne.protocol.html import HtmlColumnTable
from spyne.server.twisted import TwistedWebResource


HOST = '127.0.0.1'
PORT = 8000


class HelloWorldService(Service):
    @rpc(Unicode(default='World'), UnsignedInteger(default=5),
                                                    _returns=Iterable(Unicode))
    def say_hello(ctx, name, times):
        # workaround for Python2's lacking of nonlocal
        times = [times]

        def _cb(push):
            # This callback is called immediately after the function returns.

            if times[0] > 0:
                times[0] -= 1

                data = u'Hello, %s' % name
                print(data)

                # The object passed to the append() method is immediately
                # serialized to bytes and pushed to the response stream's
                # file-like object.
                push.append(data)

                # When a push-callback returns anything other than a deferred,
                # the response gets closed.
                return deferLater(reactor, 1, _cb, push)

        # This is Spyne's way of returning NOT_DONE_YET
        return Iterable.Push(_cb)

    @rpc(Unicode(default='World'), _returns=Iterable(Unicode))
    def say_hello_forever(ctx, name):
        def _cb(push):
            push.append(u'Hello, %s' % name)
            return deferLater(reactor, 0.1, _cb, push)

        return Iterable.Push(_cb)


if __name__=='__main__':
    application = Application([HelloWorldService],
            'spyne.examples.twisted.resource_push',
            in_protocol=HttpRpc(),
            out_protocol=HtmlColumnTable(),
        )

    resource = TwistedWebResource(application)
    site = Site(resource)

    reactor.listenTCP(PORT, site, interface=HOST)

    logging.basicConfig(level=logging.DEBUG)
    logging.getLogger('spyne.protocol.xml').setLevel(logging.DEBUG)
    logging.info("listening on: %s:%d" % (HOST,PORT))
    logging.info('wsdl is at: http://%s:%d/?wsdl' % (HOST, PORT))

    sys.exit(reactor.run())



#!/usr/bin/env python

from __future__ import absolute_import

from spyne import Service, rpc, Application, Fault
from spyne.server.null import NullServer
from spyne.util.color import G


class SomeService(Service):
    @rpc()
    def server_exception(ctx):
        raise Exception("boo!")

    @rpc()
    def server_fault(ctx):
        raise Fault("Server", "boo and you know it!")

    @rpc()
    def client_fault(ctx):
        raise Fault("Client", "zzzz...")


server = NullServer(Application([SomeService], 'spyne.examples.logging'))


if __name__ == "__main__":
    import logging
    logging.basicConfig(level=logging.DEBUG)

    logging.info(G("all fault tracebacks are logged"))
    logging.getLogger('spyne.application').setLevel(logging.DEBUG)

    try:
        server.service.server_exception()
    except:
        pass
    try:
        server.service.server_fault()
    except:
        pass
    try:
        server.service.client_fault()
    except:
        pass

    logging.info(G("client fault tracebacks are hidden"))
    logging.getLogger('spyne.application.client').setLevel(logging.CRITICAL)
    try:
        server.service.server_fault()
    except:
        pass
    try:
        server.service.client_fault()
    except:
        pass




#
# spyne - Copyright (C) Spyne contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

"""cdict (ClassDict) is a funny kind of dict that tries to return the values for
the base classes of a key when the entry for the key is not found. It is not a
generalized dictionary that can handle any type of key -- it relies on
spyne.model api to look for classes. It also assumes cdict never changes after
the first lookup.

>>> from spyne.util.cdict import cdict
>>> class A(object):
...     pass
...
>>> class B(A):
...     pass
...
>>> class C(object):
...     pass
...
>>> class D:
...     pass
...
>>> d=cdict({A: "fun", object: "base"})
>>> print d[A]
fun
>>> print d
{<class '__main__.A'>: 'fun', <type 'object'>: 'base'}
>>> print d[B]
fun
>>> print d
{<class '__main__.A'>: 'fun', <class '__main__.B'>: 'fun', <type 'object'>: 'base'}
>>> print d[C]
base
>>> print d
{<class '__main__.A'>: 'fun', <class '__main__.B'>: 'fun', <class '__main__.C'>: 'base', <type 'object'>: 'base'}
>>> print d[D]
*** KeyError: <class __main__.D at 0x8d92c0>
>>>
"""

import logging
logger = logging.getLogger(__name__)

class cdict(dict):
    def __getitem__(self, cls):
        try:
            return dict.__getitem__(self, cls)

        except KeyError as e:
            if not hasattr(cls, '__bases__'):
                cls = cls.__class__

            for b in reversed(cls.__bases__):
                try:
                    retval = self[b]
                    # this is why a cdict instance must never be modified after
                    # the first lookup
                    self[cls] = retval
                    return retval
                except KeyError:
                    pass
            raise e

    def get(self, k, d=None):
        try:
            return self[k]

        except KeyError:
            return d




#
# spyne - Copyright (C) Spyne contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

from sqlalchemy import sql
from sqlalchemy.ext.compiler import compiles
from sqlalchemy.sql.type_api import UserDefinedType


class PGGeometry(UserDefinedType):
    """Geometry type for Postgis 2"""

    class PlainWkt:
        pass

    class PlainWkb:
        pass

    def __init__(self, geometry_type='GEOMETRY', srid=4326, dimension=2,
                                                                  format='wkt'):
        self.geometry_type = geometry_type.upper()
        self.name = 'geometry'
        self.srid = int(srid)
        self.dimension = dimension
        self.format = format

        if self.format == 'wkt':
            self.format = PGGeometry.PlainWkt
        elif self.format == 'wkb':
            self.format = PGGeometry.PlainWkb

    def get_col_spec(self):
        return '%s(%s,%d)' % (self.name, self.geometry_type, self.srid)

    def column_expression(self, col):
        if self.format is PGGeometry.PlainWkb:
            return sql.func.ST_AsBinary(col, type_=self)
        if self.format is PGGeometry.PlainWkt:
            return sql.func.ST_AsText(col, type_=self)

    def result_processor(self, dialect, coltype):
        if self.format is PGGeometry.PlainWkt:
            def process(value):
                if value is not None:
                    return value

        if self.format is PGGeometry.PlainWkb:
            def process(value):
                if value is not None:
                    return sql.func.ST_AsBinary(value, self.srid)

        return process

    def bind_expression(self, bindvalue):
        if self.format is PGGeometry.PlainWkt:
            return sql.func.ST_GeomFromText(bindvalue, self.srid)


Geometry = PGGeometry


@compiles(PGGeometry)
def compile_geometry(type_, compiler, **kw):
    return '%s(%s,%d)' % (type_.name, type_.geometry_type, type_.srid)


@compiles(PGGeometry, "sqlite")
def compile_geometry_sqlite(type_, compiler, **kw):
    return "BLOB"



# encoding: utf8
#
# spyne - Copyright (C) Spyne contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

from __future__ import print_function

import logging
logger = logging.getLogger(__name__)

from inspect import isgenerator

from lxml import etree
from lxml.etree import LxmlSyntaxError

from spyne import ProtocolContext, BODY_STYLE_WRAPPED, ByteArray, File, Array
from spyne.util import Break, coroutine
from spyne.protocol import ProtocolMixin

from spyne.protocol.cloth.to_parent import ToParentMixin
from spyne.protocol.cloth.to_cloth import ToClothMixin
from spyne.util.six import BytesIO
from spyne.util.color import R, B
from spyne.util.tlist import tlist


class XmlClothProtocolContext(ProtocolContext):
    def __init__(self, parent, transport, type=None):
        super(XmlClothProtocolContext, self).__init__(parent, transport, type)

        self.inst_stack = tlist([], tuple)
        self.prot_stack = tlist([], ProtocolMixin)
        self.doctype_written = False


class XmlCloth(ToParentMixin, ToClothMixin):
    mime_type = 'text/xml'
    HtmlMicroFormat = None

    def __init__(self, app=None, encoding='utf8', doctype=None,
                      mime_type=None, ignore_uncap=False, ignore_wrappers=False,
                                cloth=None, cloth_parser=None, polymorphic=True,
                         strip_comments=True, use_ns=None, skip_root_tag=False):

        super(XmlCloth, self).__init__(app=app, mime_type=mime_type,
                     ignore_uncap=ignore_uncap, ignore_wrappers=ignore_wrappers,
                                                        polymorphic=polymorphic)

        self._init_cloth(cloth, cloth_parser, strip_comments)
        self.developer_mode = False
        self.encoding = encoding
        self.default_method = 'xml'
        self.doctype = doctype
        self.use_ns = use_ns
        self.skip_root_tag = skip_root_tag

    def get_context(self, parent, transport):
        return XmlClothProtocolContext(parent, transport)

    def serialize(self, ctx, message):
        """Uses ``ctx.out_object``, ``ctx.out_header`` or ``ctx.out_error`` to
        set ``ctx.out_body_doc``, ``ctx.out_header_doc`` and
        ``ctx.out_document`` as an ``lxml.etree._Element instance``.

        Not meant to be overridden.
        """

        assert message in (self.REQUEST, self.RESPONSE)

        self.event_manager.fire_event('before_serialize', ctx)

        if ctx.out_stream is None:
            ctx.out_stream = BytesIO()
            logger.debug("%r %d", ctx.out_stream, id(ctx.out_stream))

        if ctx.out_error is not None:
            # All errors at this point must be Fault subclasses.
            inst = ctx.out_error
            cls = inst.__class__
            name = cls.get_type_name()

            if self.developer_mode:
                # FIXME: the eff is this?
                ctx.out_object = (inst,)

                retval = self._incgen(ctx, cls, inst, name)
            else:
                with self.docfile(ctx.out_stream, encoding=self.encoding) as xf:
                    retval = self.to_parent(ctx, cls, inst, xf, name)

        else:
            assert message is self.RESPONSE
            result_class = ctx.descriptor.out_message

            name = result_class.get_type_name()
            if ctx.descriptor.body_style == BODY_STYLE_WRAPPED:
                if self.ignore_wrappers:
                    result_inst = ctx.out_object[0]
                    while result_class.Attributes._wrapper and \
                                              len(result_class._type_info) == 1:
                        result_class, = result_class._type_info.values()

                else:
                    result_inst = result_class()

                    for i, attr_name in enumerate(
                                        result_class._type_info.keys()):
                        setattr(result_inst, attr_name, ctx.out_object[i])

            else:
                result_inst, = ctx.out_object

            retval = self._incgen(ctx, result_class, result_inst, name)

        self.event_manager.fire_event('after_serialize', ctx)

        return retval

    def create_out_string(self, ctx, charset=None):
        """Sets an iterable of string fragments to ctx.out_string if the output
        is a StringIO object, which means we're run by a sync framework. Async
        frameworks have the out_stream write directly to the output stream so
        out_string should not be used.
        """

        if isinstance(ctx.out_stream, BytesIO):
            ctx.out_string = [ctx.out_stream.getvalue()]

    @coroutine
    def _incgen(self, ctx, cls, inst, name):
        """Entry point to the (stack of) XmlCloth-based protocols.

        Not supposed to be overridden.
        """

        if name is None:
            name = cls.get_type_name()

        try:
            with self.docfile(ctx.out_stream, encoding=self.encoding) as xf:
                ctx.outprot_ctx.doctype_written = False
                ctx.protocol.prot_stack = tlist([], ProtocolMixin)
                ret = self.subserialize(ctx, cls, inst, xf, name)

                if isgenerator(ret):  # Poor man's yield from
                    try:
                        while True:
                            sv2 = (yield)
                            ret.send(sv2)

                    except Break as b:
                        try:
                            ret.throw(b)
                        except StopIteration:
                            pass

        except LxmlSyntaxError as e:
            if e.msg == 'no content written':
                pass
            else:
                raise

    def docfile(self, *args, **kwargs):
        logger.debug("Starting file with %r %r", args, kwargs)
        return etree.xmlfile(*args, **kwargs)

    def _get_doctype(self, cloth):
        if self.doctype is not None:
            return self.doctype

        if cloth is not None:
            return cloth.getroottree().docinfo.doctype

        if self._root_cloth is not None:
            return self._root_cloth.getroottree().docinfo.doctype

        if self._cloth is not None:
            return self._cloth.getroottree().docinfo.doctype

    def write_doctype(self, ctx, parent, cloth=None):
        dt = self._get_doctype(cloth)
        if dt is None:
            return

        parent.write_doctype(dt)
        ctx.outprot_ctx.doctype_written = True
        logger.debug("Doctype written as: '%s'", dt)

    @staticmethod
    def get_class_cloth(cls):
        return cls.Attributes._xml_cloth

    @staticmethod
    def get_class_root_cloth(cls):
        return cls.Attributes._xml_root_cloth

    def check_class_cloths(self, ctx, cls, inst, parent, name, **kwargs):
        c = self.get_class_root_cloth(cls)
        eltstack = getattr(ctx.protocol, 'eltstack', [])
        if c is not None and len(eltstack) == 0 and not (eltstack[-1] is c):
            if not ctx.outprot_ctx.doctype_written:
                self.write_doctype(ctx, parent, c)

            logger.debug("to object root cloth")
            return True, self.to_root_cloth(ctx, cls, inst, c, parent, name,
                                                                       **kwargs)
        c = self.get_class_cloth(cls)
        if c is not None:
            if not ctx.outprot_ctx.doctype_written:
                self.write_doctype(ctx, parent, c)

            logger.debug("to object cloth")
            return True, self.to_parent_cloth(ctx, cls, inst, c, parent, name,
                                                                       **kwargs)
        return False, None

    @coroutine
    def subserialize(self, ctx, cls, inst, parent, name='', **kwargs):
        """Bridge between multiple XmlCloth-based protocols.

        Not supposed to be overridden.
        """

        pstack = ctx.protocol.prot_stack
        pstack.append(self)
        logger.debug("%s push prot %r. newlen: %d", R("%"), self, len(pstack))

        have_cloth = False

        cls_cloth = self.get_class_cloth(cls)
        if cls_cloth is not None:
            logger.debug("to object cloth for %s", cls.get_type_name())
            ret = self.to_parent_cloth(ctx, cls, inst, cls_cloth, parent, name)

        elif self._root_cloth is not None:
            logger.debug("to root cloth for %s", cls.get_type_name())
            ret = self.to_root_cloth(ctx, cls, inst, self._root_cloth,
                                                                   parent, name)
            have_cloth = True

        elif self._cloth is not None:
            logger.debug("to parent protocol cloth for %s", cls.get_type_name())
            ret = self.to_parent_cloth(ctx, cls, inst, self._cloth, parent,
                                                                           name)
            have_cloth = True

        else:
            logger.debug("to parent for %s", cls.get_type_name())
            ret = self.start_to_parent(ctx, cls, inst, parent, name, **kwargs)

        if isgenerator(ret):  # Poor man's yield from
            try:
                while True:
                    sv2 = (yield)
                    ret.send(sv2)

            except Break as b:
                try:
                    ret.throw(b)
                except StopIteration:
                    pass
                finally:
                    self._finalize_protocol(ctx, parent, have_cloth)
        else:
            self._finalize_protocol(ctx, parent, have_cloth)

        pstack.pop()
        logger.debug("%s pop prot  %r. newlen: %d", B("%"), self, len(pstack))

    def _finalize_protocol(self, ctx, parent, have_cloth):
        if have_cloth:
            self._close_cloth(ctx, parent)
            return

        if len(ctx.protocol.prot_stack) == 1 and len(ctx.protocol.eltstack) > 0:
            self._close_cloth(ctx, parent)
            return

    @staticmethod
    def _gen_tagname(ns, name):
        if ns is not None:
            name = "{%s}%s" % (ns, name)
        return name

    def _gen_attrib_dict(self, inst, fti):
        attrs = {}

        for field_name, field_type in fti.attrs.items():
            ns = field_type._ns
            if ns is None:
                ns = field_type.Attributes.sub_ns

            sub_name = field_type.Attributes.sub_name
            if sub_name is None:
                sub_name = field_name

            val = getattr(inst, field_name, None)
            sub_name = self._gen_tagname(ns, sub_name)

            if issubclass(field_type.type, (ByteArray, File)):
                valstr = self.to_unicode(field_type.type, val,
                                                       self.binary_encoding)
            else:
                valstr = self.to_unicode(field_type.type, val)

            if valstr is not None:
                attrs[sub_name] = valstr

        return attrs

    def decompose_incoming_envelope(self, ctx, message):
        raise NotImplementedError("This is an output-only protocol.")



# coding: utf-8

"""Rpc test models."""

from django.core.validators import MinLengthValidator, MaxLengthValidator
from django.db import models


class FieldContainer(models.Model):

    """Test model for ``DjangoMapper``."""

    char_field = models.CharField(max_length=32, default='test')
    char_field_nullable = models.CharField(max_length=32, null=True)
    slug_field = models.SlugField(max_length=32, unique=True)
    text_field = models.TextField(default='text_field')
    email_field = models.EmailField()
    boolean_field = models.BooleanField(default=True)
    integer_field = models.IntegerField(default=1)
    positive_integer_field = models.PositiveIntegerField(default=1)
    float_field = models.FloatField(default=1)
    decimal_field = models.DecimalField(max_digits=10, decimal_places=4,
                                        default=1)
    time_field = models.TimeField(auto_now_add=True)
    date_field = models.DateField(auto_now_add=True)
    datetime_field = models.DateTimeField(auto_now_add=True)

    foreign_key = models.ForeignKey('self', null=True,
                    related_name='related_containers', on_delete=models.CASCADE)
    one_to_one_field = models.OneToOneField('self', null=True,
                                                       on_delete=models.CASCADE)

    custom_foreign_key = models.ForeignKey('RelatedFieldContainer', null=True,
               related_name='related_fieldcontainers', on_delete=models.CASCADE)
    custom_one_to_one_field = models.OneToOneField('RelatedFieldContainer',
                                            null=True, on_delete=models.CASCADE)

    url_field = models.URLField(default='http://example.com')
    file_field = models.FileField(upload_to='test_file', null=True)
    excluded_field = models.CharField(max_length=32, default='excluded')
    blank_field = models.CharField(max_length=32, blank=True)
    length_validators_field = models.CharField(
        max_length=32, null=True, validators=[MinLengthValidator(3),
                                              MaxLengthValidator(10)])


class RelatedFieldContainer(models.Model):

    """Related container model to test related fields."""

    id = models.CharField(max_length=30, primary_key=True)


class User(models.Model):

    """Model for tests of relation field mapper."""

    name = models.CharField(max_length=50)


class UserProfile(models.Model):

    """Related model for tests of relation field mapper."""

    user = models.ForeignKey(User, on_delete=models.CASCADE)
    data = models.CharField(max_length=50)



#!/usr/bin/env python
# encoding: utf8
#
# Copyright © Burak Arslan <burak at arskom dot com dot tr>,
#             Arskom Ltd. http://www.arskom.com.tr
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    1. Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#    2. Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#    3. Neither the name of the owner nor the names of its contributors may be
#       used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#


"""
This is a simple HelloWorld example to show the basics of writing a Http api
using Spyne. Here's a sample:

$ curl http://localhost:8000/say_hello?name=Dave\&times=3
["Hello, Dave", "Hello, Dave", "Hello, Dave"]
"""


import sys
import logging

from spyne import Application, rpc, Service, Iterable, UnsignedInteger, \
    String

from spyne.protocol.json import JsonDocument
from spyne.protocol.http import HttpRpc
from spyne.server.wsgi import WsgiApplication
from spyne.util.cherry import cherry_graft_and_start


class HelloWorldService(Service):
    @rpc(String, UnsignedInteger, _returns=Iterable(String))
    def say_hello(ctx, name, times):
        """
        Docstrings for service methods do appear as documentation in the
        interface documents. <b>What fun!</b>

        :param name: The name to say hello to
        :param times: The number of times to say hello

        :returns: An array of 'Hello, <name>' strings, repeated <times> times.
        """

        for i in range(times):
            yield 'Hello, %s' % name


if __name__ == '__main__':
    # Python daemon boilerplate
    logging.basicConfig(level=logging.DEBUG)

    # Instantiate the application by giving it:
    #   * The list of services it should wrap,
    #   * A namespace string.
    #   * An input protocol.
    #   * An output protocol.
    application = Application([HelloWorldService], 'spyne.examples.hello.http',
          # The input protocol is set as HttpRpc to make our service easy to
          # call. Input validation via the 'soft' engine is enabled. (which is
          # actually the the only validation method for HttpRpc.)
          in_protocol=HttpRpc(validator='soft'),

          # The ignore_wrappers parameter to JsonDocument simplifies the reponse
          # dict by skipping outer response structures that are redundant when
          # the client knows what object to expect.
          out_protocol=JsonDocument(ignore_wrappers=True),
      )

    # Now that we have our application, we must wrap it inside a transport.
    # In this case, we use Spyne's standard Wsgi wrapper. Spyne supports
    # popular Http wrappers like Twisted, Django, Pyramid, etc. as well as
    # a ZeroMQ (REQ/REP) wrapper.
    wsgi_application = WsgiApplication(application)

    sys.exit(cherry_graft_and_start(wsgi_application))



#!/usr/bin/env python

# !!!!!!!!!1111!!!!!!!!!! ATTENTION !!!!!11111!!!!!!!!!!!!!
# This example is not finished. It doesn't work yet because
# <union> is not implemented
# !!11111!!!!!!!11111!!!! ATTENTION !!!!!one!!!!!11111!!!!!

from __future__ import print_function


EXAMPLE_DOCUMENT = """
<wsse:Security xmlns:wsse="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd" soapenv:mustUnderstand="1">
    <wsu:Timestamp xmlns:wsu="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd" wsu:Id="Timestamp-15452452">
        <wsu:Created>2016-02-01T10:14:54.517Z</wsu:Created>
        <wsu:Expires>2016-02-01T10:19:54.517Z</wsu:Expires>
    </wsu:Timestamp>
    <ds:Signature xmlns:ds="http://www.w3.org/2000/09/xmldsig#" Id="Signature-2088192064">
        <ds:SignedInfo>
            <ds:CanonicalizationMethod Algorithm="http://www.w3.org/2001/10/xml-exc-c14n#" />
            <ds:SignatureMethod Algorithm="http://www.w3.org/2000/09/xmldsig#rsa-sha1" />
            <ds:Reference URI="#Id-1052429873">
                <ds:Transforms>
                    <ds:Transform Algorithm="http://www.w3.org/2001/10/xml-exc-c14n#" />
                </ds:Transforms>
                <ds:DigestMethod Algorithm="http://www.w3.org/2000/09/xmldsig#sha1" />
                <ds:DigestValue>...</ds:DigestValue>
            </ds:Reference>
            <ds:Reference URI="#Timestamp-15452452">
                <ds:Transforms>
                    <ds:Transform Algorithm="http://www.w3.org/2001/10/xml-exc-c14n#" />
                </ds:Transforms>
                <ds:DigestMethod Algorithm="http://www.w3.org/2000/09/xmldsig#sha1" />
                <ds:DigestValue>...</ds:DigestValue>
            </ds:Reference>
        </ds:SignedInfo>
        <ds:SignatureValue>
...
        </ds:SignatureValue>
        <ds:KeyInfo Id="KeyId-8475839474">
            <wsse:SecurityTokenReference xmlns:wsu="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd" wsu:Id="STRId-680050181">
                <wsse:KeyIdentifier EncodingType="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-soap-message-security-1.0#Base64Binary" ValueType="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509SubjectKeyIdentifier">...</wsse:KeyIdentifier>
            </wsse:SecurityTokenReference>
        </ds:KeyInfo>
    </ds:Signature>
</wsse:Security>
"""

class NS:
    WSSE = "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd"
    WSU = "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd"
    DS = "http://www.w3.org/2000/09/xmldsig#"
    XSD = "http://www.w3.org/2001/XMLSchema"
    XML = "http://www.w3.org/XML/1998/namespace"

files = {
    NS.WSSE: "wsse.xsd",
    NS.WSU: "wsu.xsd",
    NS.DS: "ds.xsd",
    NS.XSD: "xsd.xsd",
    NS.XML: "xml.xsd",
}


from os.path import isfile
from datetime import datetime, timedelta

from spyne import Service
from spyne.util.xml import parse_schema_file


for ns, fn in files.items():
    if not isfile(fn):
        print("Missing file", fn)
        raise Exception("Please run 'make' in this script's directory to fetch"
                        "schema files before running this example")


import logging
logging.basicConfig(level=logging.DEBUG)


wsse = parse_schema_file(files[NS.WSSE], files=files)
wsu = parse_schema_file(files[NS.WSU], files=files)


class InteropServiceWithHeader(Service):
    __out_header__ = Security

    @rpc(_returns=Security)
    def send_out_header(ctx):
        ctx.out_header = Security(
            timestamp=TimeStamp(
                created=datetime.now(),
                expired=datetime.now() + timedelta(days=365),
            )
        )
        ctx.out_header.dt = datetime(year=2000, month=1, day=1)
        ctx.out_header.f = 3.141592653

        return ctx.out_header




#
# spyne - Copyright (C) Spyne contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#

"""The ``spyne.protocol.yaml`` package contains the Yaml-related protocols.
Currently, only :class:`spyne.protocol.yaml.YamlDocument` is supported.

Initially released in 2.10.0-rc.
"""

from __future__ import absolute_import

import logging
logger = logging.getLogger(__name__)

from spyne import ValidationError
from spyne.util import six
from spyne.model.binary import BINARY_ENCODING_BASE64
from spyne.model.primitive import Boolean
from spyne.model.primitive import Integer
from spyne.model.primitive import Double
from spyne.model.fault import Fault
from spyne.protocol.dictdoc import HierDictDocument

import yaml

from yaml.parser import ParserError
try:
    from yaml import CLoader as Loader
    from yaml import CDumper as Dumper
    from yaml import CSafeLoader as SafeLoader
    from yaml import CSafeDumper as SafeDumper

except ImportError:
    from yaml import Loader
    from yaml import Dumper
    from yaml import SafeLoader
    from yaml import SafeDumper


NON_NUMBER_TYPES = tuple({list, dict, six.text_type, six.binary_type})


class YamlDocument(HierDictDocument):
    """An implementation of the Yaml protocol that uses the PyYaml package.
    See ProtocolBase ctor docstring for its arguments. Yaml-specific arguments
    follow:

    :param safe: Use ``safe_dump`` instead of ``dump`` and ``safe_load`` instead
    of ``load``. This is not a security feature, search for 'safe_dump' in
    http://www.pyyaml.org/wiki/PyYAMLDocumentation
    :param kwargs: See the yaml documentation in ``load, ``safe_load``, ``dump``
    or ``safe_dump`` depending on whether you use yaml as an input or output
    protocol.

    For the output case, Spyne sets ``default_flow_style=False`` and
    ``indent=4`` by default.
    """

    mime_type = 'text/yaml'

    type = set(HierDictDocument.type)
    type.add('yaml')

    text_based = True

    default_binary_encoding = BINARY_ENCODING_BASE64

    # for test classes
    _decimal_as_string = True

    def __init__(self, app=None, validator=None, mime_type=None,
                                        ignore_uncap=False,
                                        # DictDocument specific
                                        ignore_wrappers=True,
                                        complex_as=dict,
                                        ordered=False,
                                        polymorphic=False,
                                        # YamlDocument specific
                                        safe=True,
                                        encoding='UTF-8',
                                        allow_unicode=True,
                                        **kwargs):

        super(YamlDocument, self).__init__(app, validator, mime_type,
                ignore_uncap, ignore_wrappers, complex_as, ordered, polymorphic)

        self._from_unicode_handlers[Double] = self._ret_number
        self._from_unicode_handlers[Boolean] = self._ret_bool
        self._from_unicode_handlers[Integer] = self._ret_number

        self._to_unicode_handlers[Double] = self._ret
        self._to_unicode_handlers[Boolean] = self._ret
        self._to_unicode_handlers[Integer] = self._ret

        loader = Loader
        dumper = Dumper
        if safe:
            loader = SafeLoader
            dumper =  SafeDumper

        self.in_kwargs = dict(kwargs)
        self.out_kwargs = dict(kwargs)

        self.in_kwargs['Loader'] = loader
        self.out_kwargs['Dumper'] = dumper

        loader.add_constructor('tag:yaml.org,2002:python/unicode',
                                                                _unicode_loader)

        self.out_kwargs['encoding'] = encoding
        self.out_kwargs['allow_unicode'] = allow_unicode

        if not 'indent' in self.out_kwargs:
            self.out_kwargs['indent'] = 4

        if not 'default_flow_style' in self.out_kwargs:
            self.out_kwargs['default_flow_style'] = False

    def _ret(self, _, value):
        return value

    def _ret_number(self, _, value):
        if isinstance(value, NON_NUMBER_TYPES):
            raise ValidationError(value)
        if value in (True, False):
            return int(value)
        return value

    def _ret_bool(self, _, value):
        if value is None or value in (True, False):
            return value
        raise ValidationError(value)

    def create_in_document(self, ctx, in_string_encoding=None):
        """Sets ``ctx.in_document``  using ``ctx.in_string``."""

        if in_string_encoding is None:
            in_string_encoding = 'UTF-8'

        try:
            try:
                s = b''.join(ctx.in_string).decode(in_string_encoding)
            except TypeError:
                s = ''.join(ctx.in_string)

            ctx.in_document = yaml.load(s, **self.in_kwargs)

        except ParserError as e:
            raise Fault('Client.YamlDecodeError', repr(e))

    def create_out_string(self, ctx, out_string_encoding='utf8'):
        """Sets ``ctx.out_string`` using ``ctx.out_document``."""

        ctx.out_string = (yaml.dump(o, **self.out_kwargs)
                                                      for o in ctx.out_document)
        if six.PY2 and out_string_encoding is not None:
            ctx.out_string = (
                yaml.dump(o, **self.out_kwargs).encode(out_string_encoding)
                                                      for o in ctx.out_document)


def _unicode_loader(loader, node):
    return node.value


def _decimal_to_bytes():
    pass


def _decimal_from_bytes():
    pass
